@@ -46,7 +46,6 @@ import qualified Data.Set as S
 import qualified Data.Text as T
 import Data.Time.Calendar (fromGregorian)
 import qualified Data.Map as M
-import Safe (headDef)
 import Text.Printf (printf)
 
 import Hledger.Utils
@@ -57,6 +56,7 @@ import Hledger.Data.Journal
 import Hledger.Data.Posting
 import Hledger.Data.Transaction
 import Hledger.Data.Errors
+import Data.Bifunctor (second)
 
 
 data BalancingOpts = BalancingOpts
@@ -93,6 +93,7 @@ defbalancingopts = BalancingOpts
 transactionCheckBalanced :: BalancingOpts -> Transaction -> [String]
 transactionCheckBalanced BalancingOpts{commodity_styles_} t = errs
   where
+    -- get real and balanced virtual postings, to be checked separately
     (rps, bvps) = foldr partitionPosting ([], []) $ tpostings t
       where
         partitionPosting p ~(l, r) = case ptype p of
@@ -100,24 +101,30 @@ transactionCheckBalanced BalancingOpts{commodity_styles_} t = errs
             BalancedVirtualPosting -> (l, p:r)
             VirtualPosting         -> (l, r)
 
-    -- check for mixed signs, detecting nonzeros at display precision
-    canonicalise = maybe id canonicaliseMixedAmount commodity_styles_
+    -- convert this posting's amount to cost,
+    -- without getting confused by redundant costs/equity postings
     postingBalancingAmount p
       | "_price-matched" `elem` map fst (ptags p) = mixedAmountStripPrices $ pamount p
       | otherwise                                 = mixedAmountCost $ pamount p
-    signsOk ps =
-      case filter (not.mixedAmountLooksZero) $ map (canonicalise.postingBalancingAmount) ps of
-        nonzeros | length nonzeros >= 2
-                   -> length (nubSort $ mapMaybe isNegativeMixedAmount nonzeros) > 1
-        _          -> True
-    (rsignsok, bvsignsok)       = (signsOk rps, signsOk bvps)
 
-    -- check for zero sum, at display precision
-    (rsumcost, bvsumcost)       = (foldMap postingBalancingAmount rps, foldMap postingBalancingAmount bvps)
-    (rsumdisplay, bvsumdisplay) = (canonicalise rsumcost, canonicalise bvsumcost)
-    (rsumok, bvsumok)           = (mixedAmountLooksZero rsumdisplay, mixedAmountLooksZero bvsumdisplay)
+    -- transaction balancedness is checked at each commodity's display precision
+    lookszero = mixedAmountLooksZero . atdisplayprecision
+      where
+        atdisplayprecision = maybe id styleAmounts $ commodity_styles_
 
-    -- generate error messages, showing amounts with their original precision
+    -- when there's multiple non-zeros, check they do not all have the same sign
+    (rsignsok, bvsignsok) = (signsOk rps, signsOk bvps)
+      where
+        signsOk ps = length nonzeros < 2 || length nonzerosigns > 1
+          where
+            nonzeros = filter (not.lookszero) $ map postingBalancingAmount ps
+            nonzerosigns = nubSort $ mapMaybe isNegativeMixedAmount nonzeros
+
+    -- check that the sum looks like zero
+    (rsumcost, bvsumcost) = (foldMap postingBalancingAmount rps, foldMap postingBalancingAmount bvps)
+    (rsumok, bvsumok) = (lookszero rsumcost, lookszero bvsumcost)
+
+    -- Generate error messages if any. Show amounts with their original precisions.
     errs = filter (not.null) [rmsg, bvmsg]
       where
         rmsg
@@ -160,10 +167,12 @@ balanceTransactionHelper ::
   -> Transaction
   -> Either String (Transaction, [(AccountName, MixedAmount)])
 balanceTransactionHelper bopts t = do
-  (t', inferredamtsandaccts) <-
-    transactionInferBalancingAmount (fromMaybe M.empty $ commodity_styles_ bopts) $
-    (if infer_balancing_costs_ bopts then transactionInferBalancingCosts else id)
-    t
+  let lbl = lbl_ "balanceTransactionHelper"
+  (t', inferredamtsandaccts) <- t
+    & (if infer_balancing_costs_ bopts then transactionInferBalancingCosts else id)
+    & dbg9With (lbl "amounts after balancing-cost-inferring".show.map showMixedAmountOneLine.transactionAmounts)
+    & transactionInferBalancingAmount (fromMaybe M.empty $ commodity_styles_ bopts)
+    <&> dbg9With (lbl "balancing amounts inferred".show.map (second showMixedAmountOneLine).snd)
   case transactionCheckBalanced bopts t' of
     []   -> Right (txnTieKnot t', inferredamtsandaccts)
     errs -> Left $ transactionBalanceError t' errs'
@@ -227,10 +236,16 @@ transactionInferBalancingAmount styles t@Transaction{tpostings=ps}
   | otherwise
       = let psandinferredamts = map inferamount ps
             inferredacctsandamts = [(paccount p, amt) | (p, Just amt) <- psandinferredamts]
-        in Right (t{tpostings=map fst psandinferredamts}, inferredacctsandamts)
+        in Right (
+           t{tpostings=map fst psandinferredamts}
+          ,inferredacctsandamts
+           -- & dbg9With (lbl "inferred".show.map (showMixedAmountOneLine.snd))
+          )
   where
+    lbl = lbl_ "transactionInferBalancingAmount"
     (amountfulrealps, amountlessrealps) = partition hasAmount (realPostings t)
     realsum = sumPostings amountfulrealps
+      -- & dbg9With (lbl "real balancing amount".showMixedAmountOneLine)
     (amountfulbvps, amountlessbvps) = partition hasAmount (balancedVirtualPostings t)
     bvsum = sumPostings amountfulbvps
 
@@ -250,7 +265,17 @@ transactionInferBalancingAmount styles t@Transaction{tpostings=ps}
               -- Inferred amounts are converted to cost.
               -- Also ensure the new amount has the standard style for its commodity
               -- (since the main amount styling pass happened before this balancing pass);
-              a' = styleMixedAmount styles . mixedAmountCost $ maNegate a
+              a' = maNegate a
+                -- & dbg9With (lbl "balancing amount".showMixedAmountOneLine)
+                & mixedAmountCost
+                -- & dbg9With (lbl "balancing amount converted to cost".showMixedAmountOneLine)
+                & styleAmounts (styles
+                                -- Needed until we switch to locally-inferred balancing precisions:
+                                -- these had hard rounding set to help with balanced-checking;
+                                -- set no rounding now to avoid excessive display precision in output
+                                & amountStylesSetRounding NoRounding
+                                & dbg9With (lbl "balancing amount styles".show))
+                & dbg9With (lbl "balancing amount styled".showMixedAmountOneLine)
 
 -- | Infer costs for this transaction's posting amounts, if needed to make
 -- the postings balance, and if permitted. This is done once for the real
@@ -302,6 +327,7 @@ transactionInferBalancingCosts t@Transaction{tpostings=ps} = t{tpostings=ps'}
 costInferrerFor :: Transaction -> PostingType -> (Posting -> Posting)
 costInferrerFor t pt = maybe id infercost inferFromAndTo
   where
+    lbl = lbl_ "costInferrerFor"
     postings     = filter ((==pt).ptype) $ tpostings t
     pcommodities = map acommodity $ concatMap (amounts . pamount) postings
     sumamounts   = amounts $ sumPostings postings  -- amounts normalises to one amount per commodity & price
@@ -326,7 +352,8 @@ costInferrerFor t pt = maybe id infercost inferFromAndTo
     infercost (fromamount, toamount) p
         | [a] <- amounts (pamount p), ptype p == pt, acommodity a == acommodity fromamount
             = p{ pamount   = mixedAmount a{aprice=Just conversionprice}
-                     , poriginal = Just $ originalPosting p }
+                  & dbg9With (lbl "inferred cost".showMixedAmountOneLine)
+               , poriginal = Just $ originalPosting p }
         | otherwise = p
       where
         -- If only one Amount in the posting list matches fromamount we can use TotalPrice.
@@ -441,19 +468,28 @@ updateTransactionB t = withRunningBalance $ \BalancingState{bsTransactions}  ->
 -- and (optional) check that all balance assertions pass.
 -- Or, return an error message (just the first error encountered).
 --
--- Assumes journalInferCommodityStyles has been called, since those
+-- Assumes journalStyleAmounts has been called, since amount styles
 -- affect transaction balancing.
 --
 -- This does multiple things at once because amount inferring, balance
 -- assignments, balance assertions and posting dates are interdependent.
+--
 journalBalanceTransactions :: BalancingOpts -> Journal -> Either String Journal
 journalBalanceTransactions bopts' j' =
   let
     -- ensure transactions are numbered, so we can store them by number
     j@Journal{jtxns=ts} = journalNumberTransactions j'
     -- display precisions used in balanced checking
-    styles = Just $ journalCommodityStyles j
+    styles = Just $
+      -- Use all the specified commodity display precisions, with hard rounding, when checking txn balancedness.
+      -- XXX Problem, those precisions will also be used when inferring balancing amounts;
+      -- it would be better to give those the precision of the amount they are balancing.
+      journalCommodityStylesWith HardRounding
+      j
     bopts = bopts'{commodity_styles_=styles}
+      -- XXX ^ The commodity directive styles and default style and inferred styles
+      -- are merged into the command line styles in commodity_styles_ - why ?
+      -- Mainly for the precisions, used during amount and cost inference and balanced checking ?
     -- balance assignments are not allowed on accounts affected by auto postings
     autopostingaccts = S.fromList . map (paccount . tmprPosting) . concatMap tmpostingrules $ jtxnmodifiers j
   in
@@ -490,6 +526,8 @@ journalBalanceTransactions bopts' j' =
         ts' <- lift $ getElems balancedtxns
         return j{jtxns=ts'}
 
+-- Before #2039: "Costs are removed, which helps eg balance-assertions.test: 15. Mix different commodities and assignments."
+
 -- | This function is called statefully on each of a date-ordered sequence of
 -- 1. fully explicit postings from already-balanced transactions and
 -- 2. not-yet-balanced transactions containing balance assignments.
@@ -497,11 +535,11 @@ journalBalanceTransactions bopts' j' =
 -- and checks balance assertions on each posting as it goes.
 -- An error will be thrown if a transaction can't be balanced
 -- or if an illegal balance assignment is found (cf checkIllegalBalanceAssignment).
--- Transaction prices are removed, which helps eg balance-assertions.test: 15. Mix different commodities and assignments.
 -- This stores the balanced transactions in case 2 but not in case 1.
 balanceTransactionAndCheckAssertionsB :: Either Posting Transaction -> Balancing s ()
 balanceTransactionAndCheckAssertionsB (Left p@Posting{}) =
-  -- update the account's running balance and check the balance assertion if any
+  -- Update the account's running balance and check the balance assertion if any.
+  -- Note, cost is ignored when checking balance assertions, currently.
   void . addAmountAndCheckAssertionB $ postingStripPrices p
 balanceTransactionAndCheckAssertionsB (Right t@Transaction{tpostings=ps}) = do
   -- make sure we can handle the balance assignments
@@ -513,7 +551,7 @@ balanceTransactionAndCheckAssertionsB (Right t@Transaction{tpostings=ps}) = do
   ps' <- ps
     & zip [1..]                 -- attach original positions
     & sortOn (postingDate.snd)  -- sort by date
-    & mapM (addOrAssignAmountAndCheckAssertionB)  -- infer amount, check assertion on each one
+    & mapM addOrAssignAmountAndCheckAssertionB  -- infer amount, check assertion on each one
     <&> sortOn fst              -- restore original order
     <&> map snd                 -- discard positions
 
@@ -577,7 +615,7 @@ addAmountAndCheckAssertionB p = return p
 -- are ignored; if it is total, they will cause the assertion to fail.
 checkBalanceAssertionB :: Posting -> MixedAmount -> Balancing s ()
 checkBalanceAssertionB p@Posting{pbalanceassertion=Just (BalanceAssertion{baamount,batotal})} actualbal =
-    forM_ (baamount : otheramts) $ \amt -> checkBalanceAssertionOneCommodityB p amt actualbal
+  forM_ (baamount : otheramts) $ \amt -> checkBalanceAssertionOneCommodityB p amt actualbal
   where
     assertedcomm = acommodity baamount
     otheramts | batotal   = map (\a -> a{aquantity=0}) . amountsRaw
@@ -586,15 +624,17 @@ checkBalanceAssertionB p@Posting{pbalanceassertion=Just (BalanceAssertion{baamou
 checkBalanceAssertionB _ _ = return ()
 
 -- | Does this (single commodity) expected balance match the amount of that
--- commodity in the given (multicommodity) actual balance ? If not, returns a
--- balance assertion failure message based on the provided posting.  To match,
--- the amounts must be exactly equal (display precision is ignored here).
+-- commodity in the given (multicommodity) actual balance, ignoring costs ?
+-- If not, returns a balance assertion failure message based on the provided posting.
+-- To match, the amounts must be exactly equal (display precision is ignored here).
 -- If the assertion is inclusive, the expected amount is compared with the account's
 -- subaccount-inclusive balance; otherwise, with the subaccount-exclusive balance.
 checkBalanceAssertionOneCommodityB :: Posting -> Amount -> MixedAmount -> Balancing s ()
-checkBalanceAssertionOneCommodityB p@Posting{paccount=assertedacct} assertedamt actualbal = do
+checkBalanceAssertionOneCommodityB p@Posting{paccount=assertedacct} assertedcommbal actualbal = do
   let isinclusive = maybe False bainclusive $ pbalanceassertion p
   let istotal     = maybe False batotal     $ pbalanceassertion p
+  -- mstyles <- R.reader bsStyles
+  -- let styled = maybe id styleAmounts mstyles
   actualbal' <-
     if isinclusive
     then
@@ -607,50 +647,84 @@ checkBalanceAssertionOneCommodityB p@Posting{paccount=assertedacct} assertedamt
           bsBalances
     else return actualbal
   let
-    assertedcomm    = acommodity assertedamt
-    actualbalincomm = headDef nullamt . amountsRaw . filterMixedAmountByCommodity assertedcomm $ actualbal'
+    assertedcomm = acommodity assertedcommbal
+
+    -- The asserted single-commodity balance, without cost
+    assertedcommbalcostless = amountStripCost assertedcommbal
+
+    -- The balance in this commodity, from the current multi-commodity running balance at this point.
+    -- This is unnormalised, and could include one or more different costs.
+    actualcommbal           = filterMixedAmountByCommodity assertedcomm $ actualbal'
+
+    -- The above balance without costs, as a single Amount (Amount's + discards costs).
+    actualcommbalcostless   = sum $ amountsRaw actualcommbal
+
+    -- test the assertion
     pass =
-      aquantity
-        -- traceWith (("asserted:"++).showAmountDebug)
-        assertedamt ==
-      aquantity
-        -- traceWith (("actual:"++).showAmountDebug)
-        actualbalincomm
+      aquantity assertedcommbalcostless
+      ==
+      aquantity actualcommbalcostless
+
     errmsg = chomp $ printf (unlines
       [ "%s:",
         "%s\n",
-        "This balance assertion failed.",
-        -- "date:       %s",
-        "In account:    %s",
-        "and commodity: %s",
-        -- "display precision:  %d",
-        "this balance was asserted:     %s", -- (at display precision: %s)",
-        "but the calculated balance is: %s", -- (at display precision: %s)",
-        "a difference of:               %s",
-        "",
-        "Consider viewing this account's calculated balances to troubleshoot. Eg:",
-        "",
-        "hledger reg '%s'%s -I  # -f FILE"
+        "Balance assertion failed in %s",
+        "%s at this point, %s, ignoring costs,",
+        "the expected balance is:        %s",
+        "but the calculated balance is:  %s",
+        "(difference: %s)",
+        "To troubleshoot, check this account's running balance with assertions disabled, eg:",
+        "hledger reg -I '%s'%s"
       ])
-      (sourcePosPretty pos)
-      (textChomp ex)
-      -- (showDate $ postingDate p)
-      (if isinclusive then printf "%-30s  (including subaccounts)" acct else acct)
-      (if istotal     then printf "%-30s  (no other commodities allowed)" (T.unpack assertedcomm) else (T.unpack assertedcomm))
-      -- (asprecision $ astyle actualbalincommodity)  -- should be the standard display precision I think
-      (show $ aquantity assertedamt)
-      -- (showAmount assertedamt)
-      (show $ aquantity actualbalincomm)
-      -- (showAmount actualbalincommodity)
-      (show $ aquantity assertedamt - aquantity actualbalincomm)
-      (acct ++ if isinclusive then "" else "$")
-      (if istotal then "" else (" cur:" ++ quoteForCommandLine (T.unpack assertedcomm)))
+
+      (sourcePosPretty pos)  -- position
+      (textChomp ex)  -- journal excerpt
+      acct  -- asserted account
+      (if istotal then "Across all commodities" else "In commodity " <> assertedcommstr)  -- asserted commodity or all commodities ?
+      (if isinclusive then "including subaccounts" else "excluding subaccounts" :: String)  -- inclusive or exclusive balance asserted ?
+      (pad assertedstr)  -- asserted amount, without cost
+      (pad actualstr)    -- actual amount, without cost
+        -- <> " (with costs: " <> T.pack (showMixedAmountWith fmt actualcommbal) <> ")" -- debugging
+      diffstr  -- their difference
+      (acct ++ if isinclusive then "" else "$")  -- query matching the account(s) postings
+      (if istotal then "" else (" cur:" ++ quoteForCommandLine (T.unpack assertedcomm)))  -- query matching the commodity(ies)
+
       where
         acct = T.unpack $ paccount p
         ass = fromJust $ pbalanceassertion p  -- PARTIAL: fromJust won't fail, there is a balance assertion
         pos = baposition ass
         (_,_,_,ex) = makeBalanceAssertionErrorExcerpt p
+        assertedcommstr = if T.null assertedcomm then "\"\"" else assertedcomm
+        fmt = oneLine{displayZeroCommodity=True}
+        assertedstr = showAmountWith fmt assertedcommbalcostless
+        actualstr   = showAmountWith fmt actualcommbalcostless
+        diffstr     = showAmountWith fmt $ assertedcommbalcostless - actualcommbalcostless
+        pad = fitText (Just w) Nothing False False . T.pack where w = max (length assertedstr) (length actualstr)
+
+
   unless pass $ throwError errmsg
+{- XXX
+When the posting amount has a cost, the highlight region expands to the full line:
+
+*** Exception: Error: /Users/simon/src/hledger/2024-01-21.j:12:69:
+   | 2023-12-31 closing balances
+12 |     assets:cash:petty:saved:rent       -4.00 EUR @ 2 UAH == 0.00 EUR
+   |     ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+   |     equity:opening/closing balances                8 UAH
+
+Maybe it's better than the normal region ?
+
+*** Exception: Error: /Users/simon/src/hledger/2024-01-21.j:12:61:
+   | 2023-12-31 closing balances
+12 |     assets:cash:petty:saved:rent          -4.00 EUR == 0.00 EUR @ 3 UAH
+   |                                                     ^^^^^^^^^^^^^^^^^^^
+   |     equity:opening/closing balances        4.00 EUR
+
+If changed also check flycheck-hledger, which currently highlights the equals:
+
+    assets:cash:petty:saved:rent                  -4.00 EUR @ 2 UAH == 0.00 EUR @ 3 UAH 
+                                                                    --
+-}
 
 -- | Throw an error if this posting is trying to do an illegal balance assignment.
 checkIllegalBalanceAssignmentB :: Posting -> Balancing s ()
@@ -1005,26 +1079,26 @@ tests_Balancing =
       --
       testCase "1091a" $ do
         commodityStylesFromAmounts [
-           nullamt{aquantity=1000, astyle=AmountStyle L False (Precision 3) (Just ',') Nothing}
-          ,nullamt{aquantity=1000, astyle=AmountStyle L False (Precision 2) (Just '.') (Just (DigitGroups ',' [3]))}
+           nullamt{aquantity=1000, astyle=AmountStyle L False Nothing (Just ',') (Precision 3) NoRounding}
+          ,nullamt{aquantity=1000, astyle=AmountStyle L False (Just (DigitGroups ',' [3])) (Just '.') (Precision 2) NoRounding}
           ]
          @?=
           -- The commodity style should have period as decimal mark
           -- and comma as digit group mark.
           Right (M.fromList [
-            ("", AmountStyle L False (Precision 3) (Just '.') (Just (DigitGroups ',' [3])))
+            ("", AmountStyle L False (Just (DigitGroups ',' [3])) (Just '.') (Precision 3) NoRounding)
           ])
         -- same journal, entries in reverse order
       ,testCase "1091b" $ do
         commodityStylesFromAmounts [
-           nullamt{aquantity=1000, astyle=AmountStyle L False (Precision 2) (Just '.') (Just (DigitGroups ',' [3]))}
-          ,nullamt{aquantity=1000, astyle=AmountStyle L False (Precision 3) (Just ',') Nothing}
+           nullamt{aquantity=1000, astyle=AmountStyle L False (Just (DigitGroups ',' [3])) (Just '.') (Precision 2) NoRounding}
+          ,nullamt{aquantity=1000, astyle=AmountStyle L False Nothing (Just ',') (Precision 3) NoRounding}
           ]
          @?=
           -- The commodity style should have period as decimal mark
           -- and comma as digit group mark.
           Right (M.fromList [
-            ("", AmountStyle L False (Precision 3) (Just '.') (Just (DigitGroups ',' [3])))
+            ("", AmountStyle L False (Just (DigitGroups ',' [3])) (Just '.') (Precision 3) NoRounding)
           ])
 
      ]
