@@ -1,9 +1,7 @@
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE NoMonoLocalBinds    #-}
 {-# LANGUAGE OverloadedStrings   #-}
-{-# LANGUAGE PackageImports      #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TupleSections       #-}
 {-# LANGUAGE TypeFamilies        #-}
 {-|
 
@@ -32,6 +30,8 @@ quarterly, etc.
 
 module Hledger.Data.Dates (
   -- * Misc date handling utilities
+  fromEFDay,
+  modifyEFDay,
   getCurrentDay,
   getCurrentMonth,
   getCurrentYear,
@@ -40,7 +40,9 @@ module Hledger.Data.Dates (
   periodContainsDate,
   parsedateM,
   showDate,
+  showEFDate,
   showDateSpan,
+  showDateSpanDebug,
   showDateSpanMonthAbbrev,
   elapsedSeconds,
   prevday,
@@ -77,13 +79,14 @@ module Hledger.Data.Dates (
   daysInSpan,
 
   tests_Dates
-)
+, intervalBoundaryBefore)
 where
 
-import qualified Control.Monad.Fail as Fail (MonadFail, fail)
-import Control.Applicative (liftA2)
+import Prelude hiding (Applicative(..))
+import Control.Applicative (Applicative(..))
 import Control.Applicative.Permutations
 import Control.Monad (guard, unless)
+import qualified Control.Monad.Fail as Fail (MonadFail, fail)
 import Data.Char (digitToInt, isDigit, ord)
 import Data.Default (def)
 import Data.Foldable (asum)
@@ -106,7 +109,6 @@ import Safe (headMay, lastMay, maximumMay, minimumMay)
 import Text.Megaparsec
 import Text.Megaparsec.Char (char, char', digitChar, string, string')
 import Text.Megaparsec.Char.Lexer (decimal, signed)
-import Text.Megaparsec.Custom (customErrorBundlePretty)
 import Text.Printf (printf)
 
 import Hledger.Data.Types
@@ -121,11 +123,19 @@ instance Show DateSpan where
 showDate :: Day -> Text
 showDate = T.pack . show
 
+showEFDate :: EFDay -> Text
+showEFDate = showDate . fromEFDay
+
 -- | Render a datespan as a display string, abbreviating into a
 -- compact form if possible.
+-- Warning, hides whether dates are Exact or Flex.
 showDateSpan :: DateSpan -> Text
 showDateSpan = showPeriod . dateSpanAsPeriod
 
+-- | Show a DateSpan with its begin/end dates, exact or flex.
+showDateSpanDebug :: DateSpan -> String
+showDateSpanDebug (DateSpan b e)= "DateSpan (" <> show b <> ") (" <> show e <> ")"
+
 -- | Like showDateSpan, but show month spans as just the abbreviated month name
 -- in the current locale.
 showDateSpanMonthAbbrev :: DateSpan -> Text
@@ -147,41 +157,50 @@ elapsedSeconds :: Fractional a => UTCTime -> UTCTime -> a
 elapsedSeconds t1 = realToFrac . diffUTCTime t1
 
 spanStart :: DateSpan -> Maybe Day
-spanStart (DateSpan d _) = d
+spanStart (DateSpan d _) = fromEFDay <$> d
 
 spanEnd :: DateSpan -> Maybe Day
-spanEnd (DateSpan _ d) = d
+spanEnd (DateSpan _ d) = fromEFDay <$> d
+
+spanStartDate :: DateSpan -> Maybe EFDay
+spanStartDate (DateSpan d _) = d
+
+spanEndDate :: DateSpan -> Maybe EFDay
+spanEndDate (DateSpan _ d) = d
 
 spanStartYear :: DateSpan -> Maybe Year
-spanStartYear (DateSpan d _) = fmap (first3 . toGregorian) d
+spanStartYear (DateSpan d _) = fmap (first3 . toGregorian . fromEFDay) d
 
 spanEndYear :: DateSpan -> Maybe Year
-spanEndYear (DateSpan d _) = fmap (first3 . toGregorian) d
+spanEndYear (DateSpan d _) = fmap (first3 . toGregorian. fromEFDay) d
 
 -- | Get the 0-2 years mentioned explicitly in a DateSpan.
 spanYears :: DateSpan -> [Year]
-spanYears (DateSpan ma mb) = mapMaybe (fmap (first3 . toGregorian)) [ma,mb]
+spanYears (DateSpan ma mb) = mapMaybe (fmap (first3 . toGregorian. fromEFDay)) [ma,mb]
 
 -- might be useful later: http://en.wikipedia.org/wiki/Allen%27s_interval_algebra
 
 -- | Get overall span enclosing multiple sequentially ordered spans.
+-- The start and end date will be exact or flexible depending on
+-- the first span's start date and last span's end date.
 spansSpan :: [DateSpan] -> DateSpan
-spansSpan spans = DateSpan (spanStart =<< headMay spans) (spanEnd =<< lastMay spans)
+spansSpan spans = DateSpan (spanStartDate =<< headMay spans) (spanEndDate =<< lastMay spans)
 
--- | Split a DateSpan into consecutive whole spans of the specified interval
--- which fully encompass the original span (and a little more when necessary).
+-- | Split a DateSpan into consecutive exact spans of the specified Interval.
+-- If the first argument is true and the interval is Weeks, Months, Quarters or Years,
+-- the start date will be adjusted backward if needed to nearest natural interval boundary
+-- (a monday, first of month, first of quarter or first of year).
 -- If no interval is specified, the original span is returned.
 -- If the original span is the null date span, ie unbounded, the null date span is returned.
 -- If the original span is empty, eg if the end date is <= the start date, no spans are returned.
 --
---
 -- ==== Examples:
--- >>> let t i y1 m1 d1 y2 m2 d2 = splitSpan i $ DateSpan (Just $ fromGregorian y1 m1 d1) (Just $ fromGregorian y2 m2 d2)
+-- >>> let t i y1 m1 d1 y2 m2 d2 = splitSpan True i $ DateSpan (Just $ Flex $ fromGregorian y1 m1 d1) (Just $ Flex $ fromGregorian y2 m2 d2)
 -- >>> t NoInterval 2008 01 01 2009 01 01
 -- [DateSpan 2008]
 -- >>> t (Quarters 1) 2008 01 01 2009 01 01
 -- [DateSpan 2008Q1,DateSpan 2008Q2,DateSpan 2008Q3,DateSpan 2008Q4]
--- >>> splitSpan (Quarters 1) nulldatespan
+-- >>> splitSpan True (Quarters 1) nulldatespan
 -- [DateSpan ..]
 -- >>> t (Days 1) 2008 01 01 2008 01 01  -- an empty datespan
 -- []
@@ -206,53 +225,65 @@ spansSpan spans = DateSpan (spanStart =<< headMay spans) (spanEnd =<< lastMay sp
 -- >>> t (DayOfYear 11 29) 2011 12 01 2012 12 15
 -- [DateSpan 2011-11-29..2012-11-28,DateSpan 2012-11-29..2013-11-28]
 --
-splitSpan :: Interval -> DateSpan -> [DateSpan]
-splitSpan _ (DateSpan Nothing Nothing) = [DateSpan Nothing Nothing]
-splitSpan _ ds | isEmptySpan ds = []
-splitSpan _ ds@(DateSpan (Just s) (Just e)) | s == e = [ds]
-splitSpan NoInterval      ds = [ds]
-splitSpan (Days n)        ds = splitspan startofday     addDays n                    ds
-splitSpan (Weeks n)       ds = splitspan startofweek    addDays (7*n)                ds
-splitSpan (Months n)      ds = splitspan startofmonth   addGregorianMonthsClip n     ds
-splitSpan (Quarters n)    ds = splitspan startofquarter addGregorianMonthsClip (3*n) ds
-splitSpan (Years n)       ds = splitspan startofyear    addGregorianYearsClip n      ds
-splitSpan (DayOfMonth n)  ds = splitspan (nthdayofmonthcontaining n)  addGregorianMonthsClip 1 ds
-splitSpan (DayOfYear m n) ds = splitspan (nthdayofyearcontaining m n) addGregorianYearsClip 1 ds
-splitSpan (WeekdayOfMonth n wd) ds = splitspan (nthweekdayofmonthcontaining n wd) advancemonths 1 ds
+splitSpan :: Bool -> Interval -> DateSpan -> [DateSpan]
+splitSpan _ _ (DateSpan Nothing Nothing) = [DateSpan Nothing Nothing]
+splitSpan _ _ ds | isEmptySpan ds = []
+splitSpan _ _ ds@(DateSpan (Just s) (Just e)) | s == e = [ds]
+splitSpan _ NoInterval ds = [ds]
+splitSpan _ (Days n) ds = splitspan id addDays n                    ds
+splitSpan adjust (Weeks n)    ds = splitspan (if adjust then startofweek    else id) addDays (7*n)                ds
+splitSpan adjust (Months n)   ds = splitspan (if adjust then startofmonth   else id) addGregorianMonthsClip n     ds
+splitSpan adjust (Quarters n) ds = splitspan (if adjust then startofquarter else id) addGregorianMonthsClip (3*n) ds
+splitSpan adjust (Years n)    ds = splitspan (if adjust then startofyear    else id) addGregorianYearsClip n      ds
+splitSpan _ (DayOfMonth dom)  ds = splitspan (nthdayofmonthcontaining dom) (addGregorianMonthsToMonthday dom) 1 ds
+splitSpan _ (DayOfYear m n)   ds = splitspan (nthdayofyearcontaining m n) (addGregorianYearsClip) 1 ds
+splitSpan _ (WeekdayOfMonth n wd) ds = splitspan (nthweekdayofmonthcontaining n wd) advancemonths 1 ds
   where
     advancemonths 0 = id
     advancemonths w = advancetonthweekday n wd . startofmonth . addGregorianMonthsClip w
-splitSpan (DaysOfWeek [])         ds = [ds]
-splitSpan (DaysOfWeek days@(n:_)) ds = spansFromBoundaries e bdrys
+splitSpan _ (DaysOfWeek [])         ds = [ds]
+splitSpan _ (DaysOfWeek days@(n:_)) ds = spansFromBoundaries e bdrys
   where
     (s, e) = dateSpanSplitLimits (nthdayofweekcontaining n) nextday ds
     bdrys = concatMap (flip map starts . addDays) [0,7..]
     -- The first representative of each weekday
     starts = map (\d -> addDays (toInteger $ d - n) $ nthdayofweekcontaining n s) days
 
--- Split the given span using the provided helper functions:
--- start is applied to the span's start date to get the first sub-span's start date
--- addInterval is applied to an integer n (multiplying it by mult) and the span's start date to get the nth sub-span's start date
+-- Like addGregorianMonthsClip, add one month to the given date, clipping when needed
+-- to fit it within the next month's length. But also, keep a target day of month in mind,
+-- and revert to that or as close to it as possible in subsequent longer months.
+-- Eg, using it to step through 31sts gives 1/31, 2/28, 3/31, 4/30, 5/31..
+addGregorianMonthsToMonthday :: MonthDay -> Integer -> Day -> Day
+addGregorianMonthsToMonthday dom n d =
+  let (y,m,_) = toGregorian $ addGregorianMonthsClip n d
+  in fromGregorian y m dom
+
+-- Split the given span into exact spans using the provided helper functions:
+-- 1. The start function is applied to the span's start date to get the first sub-span's start date.
+-- 2. The addInterval function is used to calculate the subsequent spans' start dates,
+-- possibly with stride increased by the mult multiplier.
+-- It should adapt to spans of varying length, eg if splitting on "every 31st of month"
+-- addInterval should adjust to 28/29/30 in short months but return to 31 in the long months.
 splitspan :: (Day -> Day) -> (Integer -> Day -> Day) -> Int -> DateSpan -> [DateSpan]
 splitspan start addInterval mult ds = spansFromBoundaries e bdrys
   where
-    (s, e) = dateSpanSplitLimits start (addInterval $ toInteger mult) ds
+    (s, e) = dateSpanSplitLimits start (addInterval (toInteger mult)) ds
     bdrys = mapM (addInterval . toInteger) [0,mult..] $ start s
 
--- | Fill in missing endpoints for calculating 'splitSpan'.
+-- | Fill in missing start/end dates for calculating 'splitSpan'.
 dateSpanSplitLimits :: (Day -> Day) -> (Day -> Day) -> DateSpan -> (Day, Day)
-dateSpanSplitLimits start _    (DateSpan (Just s) (Just e)) = (start s, e)
-dateSpanSplitLimits start next (DateSpan (Just s) Nothing)  = (start s, next $ start s)
-dateSpanSplitLimits start next (DateSpan Nothing  (Just e)) = (start e, next $ start e)
-dateSpanSplitLimits _     _    (DateSpan Nothing   Nothing) = error "dateSpanSplitLimits: Should not be nulldatespan"  -- PARTIAL: This case should have been handled in splitSpan
+dateSpanSplitLimits start _    (DateSpan (Just s) (Just e)) = (start $ fromEFDay s, fromEFDay e)
+dateSpanSplitLimits start next (DateSpan (Just s) Nothing)  = (start $ fromEFDay s, next $ start $ fromEFDay s)
+dateSpanSplitLimits start next (DateSpan Nothing  (Just e)) = (start $ fromEFDay e, next $ start $ fromEFDay e)
+dateSpanSplitLimits _     _    (DateSpan Nothing   Nothing) = error "dateSpanSplitLimits: should not be nulldatespan"  -- PARTIAL: This case should have been handled in splitSpan
 
--- | Construct a list of 'DateSpan's from a list of boundaries, which fit within a given range.
+-- | Construct a list of exact 'DateSpan's from a list of boundaries, which fit within a given range.
 spansFromBoundaries :: Day -> [Day] -> [DateSpan]
-spansFromBoundaries e bdrys = zipWith (DateSpan `on` Just) (takeWhile (< e) bdrys) $ drop 1 bdrys
+spansFromBoundaries e bdrys = zipWith (DateSpan `on` (Just . Exact)) (takeWhile (< e) bdrys) $ drop 1 bdrys
 
 -- | Count the days in a DateSpan, or if it is open-ended return Nothing.
 daysInSpan :: DateSpan -> Maybe Integer
-daysInSpan (DateSpan (Just d1) (Just d2)) = Just $ diffDays d2 d1
+daysInSpan (DateSpan (Just d1) (Just d2)) = Just $ diffDays (fromEFDay d2) (fromEFDay d1)
 daysInSpan _ = Nothing
 
 -- | Is this an empty span, ie closed with the end date on or before the start date ?
@@ -263,9 +294,9 @@ isEmptySpan _                            = False
 -- | Does the span include the given date ?
 spanContainsDate :: DateSpan -> Day -> Bool
 spanContainsDate (DateSpan Nothing Nothing)   _ = True
-spanContainsDate (DateSpan Nothing (Just e))  d = d < e
-spanContainsDate (DateSpan (Just b) Nothing)  d = d >= b
-spanContainsDate (DateSpan (Just b) (Just e)) d = d >= b && d < e
+spanContainsDate (DateSpan Nothing (Just e))  d = d < fromEFDay e
+spanContainsDate (DateSpan (Just b) Nothing)  d = d >= fromEFDay b
+spanContainsDate (DateSpan (Just b) (Just e)) d = d >= fromEFDay b && d < fromEFDay e
 
 -- | Does the period include the given date ?
 -- (Here to avoid import cycle).
@@ -286,7 +317,7 @@ groupByDateSpan showempty date colspans =
     groupByCols (c:cs) ps = (c, map snd matches) : groupByCols cs later
       where (matches, later) = span ((spanEnd c >) . Just . fst) ps
 
-    beforeStart = maybe (const True) (>) $ spanStart =<< headMay colspans
+    beforeStart = maybe (const False) (>) $ spanStart =<< headMay colspans
 
 -- | Calculate the intersection of a number of datespans.
 spansIntersect [] = nulldatespan
@@ -296,7 +327,7 @@ spansIntersect (d:ds) = d `spanIntersect` (spansIntersect ds)
 -- | Calculate the intersection of two datespans.
 --
 -- For non-intersecting spans, gives an empty span beginning on the second's start date:
--- >>> DateSpan (Just $ fromGregorian 2018 01 01) (Just $ fromGregorian 2018 01 03) `spanIntersect` DateSpan (Just $ fromGregorian 2018 01 03) (Just $ fromGregorian 2018 01 05)
+-- >>> DateSpan (Just $ Flex $ fromGregorian 2018 01 01) (Just $ Flex $ fromGregorian 2018 01 03) `spanIntersect` DateSpan (Just $ Flex $ fromGregorian 2018 01 03) (Just $ Flex $ fromGregorian 2018 01 05)
 -- DateSpan 2018-01-03..2018-01-02
 spanIntersect (DateSpan b1 e1) (DateSpan b2 e2) = DateSpan b e
     where
@@ -332,7 +363,7 @@ earliest (Just d1) (Just d2) = Just $ min d1 d2
 -- usual exclusive-end-date sense: beginning on the earliest, and ending on
 -- the day after the latest).
 daysSpan :: [Day] -> DateSpan
-daysSpan ds = DateSpan (minimumMay ds) (addDays 1 <$> maximumMay ds)
+daysSpan ds = DateSpan (Exact <$> minimumMay ds) (Exact . addDays 1 <$> maximumMay ds)
 
 -- | Select the DateSpan containing a given Day, if any, from a given list of
 -- DateSpans.
@@ -349,19 +380,19 @@ latestSpanContaining :: [DateSpan] -> Day -> Maybe DateSpan
 latestSpanContaining datespans = go
   where
     go day = do
-        span <- Set.lookupLT supSpan spanSet
-        guard $ spanContainsDate span day
-        return span
+        spn <- Set.lookupLT supSpan spanSet
+        guard $ spanContainsDate spn day
+        return spn
       where
         -- The smallest DateSpan larger than any DateSpan containing day.
-        supSpan = DateSpan (Just $ addDays 1 day) Nothing
+        supSpan = DateSpan (Just $ Exact $ addDays 1 day) Nothing
 
     spanSet = Set.fromList $ filter (not . isEmptySpan) datespans
 
 -- | Parse a period expression to an Interval and overall DateSpan using
 -- the provided reference date, or return a parse error.
 parsePeriodExpr
-  :: Day -> Text -> Either (ParseErrorBundle Text CustomErr) (Interval, DateSpan)
+  :: Day -> Text -> Either HledgerParseErrors (Interval, DateSpan)
 parsePeriodExpr refdate s = parsewith (periodexprp refdate <* eof) (T.toLower s)
 
 -- | Like parsePeriodExpr, but call error' on failure.
@@ -388,18 +419,19 @@ spanFromSmartDate :: Day -> SmartDate -> DateSpan
 spanFromSmartDate refdate sdate = DateSpan (Just b) (Just e)
     where
       (ry,rm,_) = toGregorian refdate
-      (b,e) = span sdate
-      span :: SmartDate -> (Day,Day)
-      span (SmartRelative n Day)                    = (addDays n refdate, addDays (n+1) refdate)
-      span (SmartRelative n Week)                   = let d = thisweek refdate in (addDays (7*n) d, addDays (7*n+7) d)
-      span (SmartRelative n Month)                  = let d = thismonth refdate in (addGregorianMonthsClip n d, addGregorianMonthsClip (n+1) d)
-      span (SmartRelative n Quarter)                = let d = thisquarter refdate in (addGregorianMonthsClip (3*n) d, addGregorianMonthsClip (3*n+3) d)
-      span (SmartRelative n Year)                   = let d = thisyear refdate in (addGregorianYearsClip n d, addGregorianYearsClip (n+1) d)
-      span (SmartAssumeStart y Nothing)             = (startofyear day, nextyear day) where day = fromGregorian y 1 1
-      span (SmartAssumeStart y (Just (m, Nothing))) = (startofmonth day, nextmonth day) where day = fromGregorian y m 1
-      span (SmartAssumeStart y (Just (m, Just d)))  = (day, nextday day) where day = fromGregorian y m d
-      span (SmartFromReference m d)                 = (day, nextday day) where day = fromGregorian ry (fromMaybe rm m) d
-      span (SmartMonth m)                           = (startofmonth day, nextmonth day) where day = fromGregorian ry m 1
+      (b,e) = span' sdate
+        where
+          span' :: SmartDate -> (EFDay, EFDay)
+          span' (SmartCompleteDate day)       = (Exact day, Exact $ nextday day)
+          span' (SmartAssumeStart y Nothing)  = (Flex $ startofyear day, Flex $ nextyear day) where day = fromGregorian y 1 1
+          span' (SmartAssumeStart y (Just m)) = (Flex $ startofmonth day, Flex $ nextmonth day) where day = fromGregorian y m 1
+          span' (SmartFromReference m d)      = (Exact day, Exact $ nextday day) where day = fromGregorian ry (fromMaybe rm m) d
+          span' (SmartMonth m)                = (Flex $ startofmonth day, Flex $ nextmonth day) where day = fromGregorian ry m 1
+          span' (SmartRelative n Day)         = (Exact $ addDays n refdate, Exact $ addDays (n+1) refdate)
+          span' (SmartRelative n Week)        = (Flex $ addDays (7*n) d, Flex $ addDays (7*n+7) d) where d = thisweek refdate
+          span' (SmartRelative n Month)       = (Flex $ addGregorianMonthsClip n d, Flex $ addGregorianMonthsClip (n+1) d) where d = thismonth refdate
+          span' (SmartRelative n Quarter)     = (Flex $ addGregorianMonthsClip (3*n) d, Flex $ addGregorianMonthsClip (3*n+3) d) where d = thisquarter refdate
+          span' (SmartRelative n Year)        = (Flex $ addGregorianYearsClip n d, Flex $ addGregorianYearsClip (n+1) d) where d = thisyear refdate
 
 -- showDay :: Day -> String
 -- showDay day = printf "%04d/%02d/%02d" y m d where (y,m,d) = toGregorian day
@@ -409,19 +441,21 @@ spanFromSmartDate refdate sdate = DateSpan (Just b) (Just e)
 fixSmartDateStr :: Day -> Text -> Text
 fixSmartDateStr d s =
   either (error' . printf "could not parse date %s %s" (show s) . show) id $  -- PARTIAL:
-  (fixSmartDateStrEither d s :: Either (ParseErrorBundle Text CustomErr) Text)
+  (fixSmartDateStrEither d s :: Either HledgerParseErrors Text)
 
 -- | A safe version of fixSmartDateStr.
-fixSmartDateStrEither :: Day -> Text -> Either (ParseErrorBundle Text CustomErr) Text
-fixSmartDateStrEither d = fmap showDate . fixSmartDateStrEither' d
+fixSmartDateStrEither :: Day -> Text -> Either HledgerParseErrors Text
+fixSmartDateStrEither d = fmap showEFDate . fixSmartDateStrEither' d
 
 fixSmartDateStrEither'
-  :: Day -> Text -> Either (ParseErrorBundle Text CustomErr) Day
+  :: Day -> Text -> Either HledgerParseErrors EFDay
 fixSmartDateStrEither' d s = case parsewith smartdateonly (T.toLower s) of
                                Right sd -> Right $ fixSmartDate d sd
                                Left e -> Left e
 
--- | Convert a SmartDate to an absolute date using the provided reference date.
+-- | Convert a SmartDate to a specific date using the provided reference date.
+-- This date will be exact or flexible depending on whether the day was
+-- specified exactly. (Missing least-significant parts produces a flex date.)
 --
 -- ==== Examples:
 -- >>> :set -XOverloadedStrings
@@ -504,24 +538,24 @@ fixSmartDateStrEither' d s = case parsewith smartdateonly (T.toLower s) of
 -- "2008-07-01"
 -- >>> t "1 week ahead"
 -- "2008-12-01"
-fixSmartDate :: Day -> SmartDate -> Day
+fixSmartDate :: Day -> SmartDate -> EFDay
 fixSmartDate refdate = fix
   where
-    fix :: SmartDate -> Day
-    fix (SmartRelative n Day)     = addDays n refdate
-    fix (SmartRelative n Week)    = addDays (7*n) $ thisweek refdate
-    fix (SmartRelative n Month)   = addGregorianMonthsClip n $ thismonth refdate
-    fix (SmartRelative n Quarter) = addGregorianMonthsClip (3*n) $ thisquarter refdate
-    fix (SmartRelative n Year)    = addGregorianYearsClip n $ thisyear refdate
-    fix (SmartAssumeStart y md)   = fromGregorian y (maybe 1 fst md) (fromMaybe 1 $ snd =<< md)
-    fix (SmartFromReference m d)  = fromGregorian ry (fromMaybe rm m) d
-    fix (SmartMonth m)            = fromGregorian ry m 1
+    fix :: SmartDate -> EFDay
+    fix (SmartCompleteDate d)     = Exact d
+    fix (SmartAssumeStart y m)    = Flex  $ fromGregorian y (fromMaybe 1 m) 1
+    fix (SmartFromReference m d)  = Exact $ fromGregorian ry (fromMaybe rm m) d
+    fix (SmartMonth m)            = Flex  $ fromGregorian ry m 1
+    fix (SmartRelative n Day)     = Exact $ addDays n refdate
+    fix (SmartRelative n Week)    = Flex  $ addDays (7*n) $ thisweek refdate
+    fix (SmartRelative n Month)   = Flex  $ addGregorianMonthsClip n $ thismonth refdate
+    fix (SmartRelative n Quarter) = Flex  $ addGregorianMonthsClip (3*n) $ thisquarter refdate
+    fix (SmartRelative n Year)    = Flex  $ addGregorianYearsClip n $ thisyear refdate
     (ry, rm, _) = toGregorian refdate
 
 prevday :: Day -> Day
 prevday = addDays (-1)
 nextday = addDays 1
-startofday = id
 
 thisweek = startofweek
 prevweek = startofweek . addDays (-7)
@@ -541,13 +575,21 @@ thisquarter = startofquarter
 startofquarter day = fromGregorian y (firstmonthofquarter m) 1
     where
       (y,m,_) = toGregorian day
-      firstmonthofquarter m = ((m-1) `div` 3) * 3 + 1
+      firstmonthofquarter m2 = ((m2-1) `div` 3) * 3 + 1
 
 thisyear = startofyear
 prevyear = startofyear . addGregorianYearsClip (-1)
 nextyear = startofyear . addGregorianYearsClip 1
 startofyear day = fromGregorian y 1 1 where (y,_,_) = toGregorian day
 
+-- Get the natural start for the given interval that falls on or before the given day,
+-- when applicable. Works for Weeks, Months, Quarters, Years, eg.
+intervalBoundaryBefore :: Interval -> Day -> Day
+intervalBoundaryBefore i d =
+  case splitSpan True i (DateSpan (Just $ Exact d) (Just $ Exact $ addDays 1 d)) of
+    (DateSpan (Just start) _:_) -> fromEFDay start
+    _ -> d
+
 -- | For given date d find year-long interval that starts on given
 -- MM/DD of year and covers it.
 -- The given MM and DD should be basically valid (1-12 & 1-31),
@@ -570,19 +612,19 @@ startofyear day = fromGregorian y 1 1 where (y,_,_) = toGregorian day
 -- >>> nthdayofyearcontaining 1 1 wed22nd
 -- 2017-01-01
 nthdayofyearcontaining :: Month -> MonthDay -> Day -> Day
-nthdayofyearcontaining m md date
+nthdayofyearcontaining m mdy date
   -- PARTIAL:
   | not (validMonth m)  = error' $ "nthdayofyearcontaining: invalid month "++show m
-  | not (validDay   md) = error' $ "nthdayofyearcontaining: invalid day "  ++show md
+  | not (validDay   mdy) = error' $ "nthdayofyearcontaining: invalid day "  ++show mdy
   | mmddOfSameYear <= date = mmddOfSameYear
   | otherwise = mmddOfPrevYear
-  where mmddOfSameYear = addDays (toInteger md-1) $ applyN (m-1) nextmonth s
-        mmddOfPrevYear = addDays (toInteger md-1) $ applyN (m-1) nextmonth $ prevyear s
+  where mmddOfSameYear = addDays (toInteger mdy-1) $ applyN (m-1) nextmonth s
+        mmddOfPrevYear = addDays (toInteger mdy-1) $ applyN (m-1) nextmonth $ prevyear s
         s = startofyear date
 
--- | For given date d find month-long interval that starts on nth day of month
--- and covers it.
--- The given day of month should be basically valid (1-31), or an error is raised.
+-- | For a given date d find the month-long period that starts on day n of a month
+-- that includes d. (It will begin on day n or either d's month or the previous month.)
+-- The given day of month should be in the range 1-31, or an error will be raised.
 --
 -- Examples: lets take 2017-11-22. Month-long intervals covering it that
 -- start on 1st-22nd of month will start in Nov. However
@@ -599,13 +641,13 @@ nthdayofyearcontaining m md date
 -- >>> nthdayofmonthcontaining 30 wed22nd
 -- 2017-10-30
 nthdayofmonthcontaining :: MonthDay -> Day -> Day
-nthdayofmonthcontaining md date
+nthdayofmonthcontaining mdy date
   -- PARTIAL:
-  | not (validDay md) = error' $ "nthdayofmonthcontaining: invalid day "  ++show md
+  | not (validDay mdy) = error' $ "nthdayofmonthcontaining: invalid day "  ++show mdy
   | nthOfSameMonth <= date = nthOfSameMonth
   | otherwise = nthOfPrevMonth
-  where nthOfSameMonth = nthdayofmonth md s
-        nthOfPrevMonth = nthdayofmonth md $ prevmonth s
+  where nthOfSameMonth = nthdayofmonth mdy s
+        nthOfPrevMonth = nthdayofmonth mdy $ prevmonth s
         s = startofmonth date
 
 -- | For given date d find week-long interval that starts on nth day of week
@@ -673,8 +715,8 @@ advancetonthweekday n wd s =
 -- -- | Parse a couple of date-time string formats to a time type.
 -- parsedatetimeM :: String -> Maybe LocalTime
 -- parsedatetimeM s = asum [
---     parseTime defaultTimeLocale "%Y/%m/%d %H:%M:%S" s,
---     parseTime defaultTimeLocale "%Y-%m-%d %H:%M:%S" s
+--     parseTimeM TruedefaultTimeLocale "%Y/%m/%d %H:%M:%S" s,
+--     parseTimeM TruedefaultTimeLocale "%Y-%m-%d %H:%M:%S" s
 --     ]
 
 -- | Try to parse a couple of date string formats:
@@ -725,11 +767,11 @@ Eg:
 YYYYMMDD is parsed as year-month-date if those parts are valid
 (>=4 digits, 1-12, and 1-31 respectively):
 >>> parsewith (smartdate <* eof) "20181201"
-Right (SmartAssumeStart 2018 (Just (12,Just 1)))
+Right (SmartCompleteDate 2018-12-01)
 
 YYYYMM is parsed as year-month-01 if year and month are valid:
 >>> parsewith (smartdate <* eof) "201804"
-Right (SmartAssumeStart 2018 (Just (4,Nothing)))
+Right (SmartAssumeStart 2018 (Just 4))
 
 With an invalid month, it's parsed as a year:
 >>> parsewith (smartdate <* eof) "201813"
@@ -788,31 +830,39 @@ validDay n = n >= 1 && n <= 31
 failIfInvalidDate :: Fail.MonadFail m => SmartDate -> m SmartDate
 failIfInvalidDate s = unless isValid (Fail.fail $ "bad smart date: " ++ show s) $> s
   where isValid = case s of
-            SmartAssumeStart y (Just (m, md)) -> isJust $ fromGregorianValid y m (fromMaybe 1 md)
-            SmartFromReference mm d           -> isJust $ fromGregorianValid 2004 (fromMaybe 1 mm) d
-            SmartMonth m                      -> validMonth m
-            _                                 -> True
+            SmartAssumeStart _ (Just m) -> validMonth m
+            SmartFromReference mm d     -> isJust $ fromGregorianValid 2004 (fromMaybe 1 mm) d
+            SmartMonth m                -> validMonth m
+            _                           -> True
+
+showBadDate :: Integer -> Int -> Int -> String
+showBadDate y m d = "bad smart date: " ++ show y ++ "-" ++ show m ++ "-" ++ show d
 
 yyyymmdd :: TextParser m SmartDate
 yyyymmdd = do
   y <- read <$> count 4 digitChar
   m <- read <$> count 2 digitChar
-  d <- optional $ read <$> count 2 digitChar
-  let date = SmartAssumeStart y $ Just (m, d)
-  failIfInvalidDate date
+  mdy <- optional $ read <$> count 2 digitChar
+  case mdy of
+      Nothing -> failIfInvalidDate $ SmartAssumeStart y (Just m)
+      Just d  -> maybe (Fail.fail $ showBadDate y m d) (return . SmartCompleteDate) $
+                   fromGregorianValid y m d
 
 ymd :: TextParser m SmartDate
 ymd = do
     y <- yearp
-    fmap (SmartAssumeStart y) . optional . try $ do
+    emd <- optional . try $ do
         sep <- datesepchar
         m <- decimal
         unless (validMonth m) $ Fail.fail ("Bad month " <> show m)
-        fmap (m,) . optional . try $ do
+        option (Left m) . try $ Right <$> do
             _ <- char sep
             d <- decimal
-            failIfInvalidDate $ SmartAssumeStart y (Just (m, Just d))
-            return d
+            maybe (Fail.fail $ showBadDate y m d) return $ fromGregorianValid y m d
+    return $ case emd of
+        Nothing          -> SmartAssumeStart y Nothing
+        Just (Left m)    -> SmartAssumeStart y (Just m)
+        Just (Right day) -> SmartCompleteDate day
 
 md :: TextParser m (Month, MonthDay)
 md = do
@@ -890,6 +940,8 @@ weekdaysp = fmap head . group . sort <$> sepBy1 weekday (string' ",")
 -- Right (DayOfYear 11 29,DateSpan 2009-01-01..)
 -- >>> p "every 11/29 from 2009"
 -- Right (DayOfYear 11 29,DateSpan 2009-01-01..)
+-- >>> p "every 11/29 since 2009"
+-- Right (DayOfYear 11 29,DateSpan 2009-01-01..)
 -- >>> p "every 2nd Thursday of month to 2009"
 -- Right (WeekdayOfMonth 2 4,DateSpan ..2008-12-31)
 -- >>> p "every 1st monday of month to 2009"
@@ -990,7 +1042,7 @@ periodexprdatespanp rdate = choice $ map try [
 -- Right DateSpan 2017
 doubledatespanp :: Day -> TextParser m DateSpan
 doubledatespanp rdate = liftA2 fromToSpan
-    (optional (string' "from" *> skipNonNewlineSpaces) *> smartdate)
+    (optional ((string' "from" <|> string' "since") *> skipNonNewlineSpaces) *> smartdate)
     (skipNonNewlineSpaces *> choice [string' "to", string "..", string "-"]
     *> skipNonNewlineSpaces *> smartdate)
   where
@@ -1013,7 +1065,7 @@ quarterdatespanp rdate = do
 
 fromdatespanp :: Day -> TextParser m DateSpan
 fromdatespanp rdate = fromSpan <$> choice
-    [ string' "from" *> skipNonNewlineSpaces *> smartdate
+    [ (string' "from" <|> string' "since") *> skipNonNewlineSpaces *> smartdate
     , smartdate <* choice [string "..", string "-"]
     ]
   where
@@ -1033,9 +1085,9 @@ justdatespanp rdate =
 nulldatespan :: DateSpan
 nulldatespan = DateSpan Nothing Nothing
 
--- | A datespan of zero length, that matches no date.
+-- | An exact datespan of zero length, that matches no date.
 emptydatespan :: DateSpan
-emptydatespan = DateSpan (Just $ addDays 1 nulldate) (Just nulldate)
+emptydatespan = DateSpan (Just $ Exact $ addDays 1 nulldate) (Just $ Exact nulldate)
 
 nulldate :: Day
 nulldate = fromGregorian 0 1 1
@@ -1045,39 +1097,39 @@ nulldate = fromGregorian 0 1 1
 
 tests_Dates = testGroup "Dates"
   [ testCase "weekday" $ do
-      splitSpan (DaysOfWeek [1..5]) (DateSpan (Just $ fromGregorian 2021 07 01) (Just $ fromGregorian 2021 07 08))
-        @?= [ (DateSpan (Just $ fromGregorian 2021 06 28) (Just $ fromGregorian 2021 06 29))
-            , (DateSpan (Just $ fromGregorian 2021 06 29) (Just $ fromGregorian 2021 06 30))
-            , (DateSpan (Just $ fromGregorian 2021 06 30) (Just $ fromGregorian 2021 07 01))
-            , (DateSpan (Just $ fromGregorian 2021 07 01) (Just $ fromGregorian 2021 07 02))
-            , (DateSpan (Just $ fromGregorian 2021 07 02) (Just $ fromGregorian 2021 07 05))
+      splitSpan False (DaysOfWeek [1..5]) (DateSpan (Just $ Exact $ fromGregorian 2021 07 01) (Just $ Exact $ fromGregorian 2021 07 08))
+        @?= [ (DateSpan (Just $ Exact $ fromGregorian 2021 06 28) (Just $ Exact $ fromGregorian 2021 06 29))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 06 29) (Just $ Exact $ fromGregorian 2021 06 30))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 06 30) (Just $ Exact $ fromGregorian 2021 07 01))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 07 01) (Just $ Exact $ fromGregorian 2021 07 02))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 07 02) (Just $ Exact $ fromGregorian 2021 07 05))
             -- next week
-            , (DateSpan (Just $ fromGregorian 2021 07 05) (Just $ fromGregorian 2021 07 06))
-            , (DateSpan (Just $ fromGregorian 2021 07 06) (Just $ fromGregorian 2021 07 07))
-            , (DateSpan (Just $ fromGregorian 2021 07 07) (Just $ fromGregorian 2021 07 08))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 07 05) (Just $ Exact $ fromGregorian 2021 07 06))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 07 06) (Just $ Exact $ fromGregorian 2021 07 07))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 07 07) (Just $ Exact $ fromGregorian 2021 07 08))
             ]
 
-      splitSpan (DaysOfWeek [1, 5]) (DateSpan (Just $ fromGregorian 2021 07 01) (Just $ fromGregorian 2021 07 08))
-        @?= [ (DateSpan (Just $ fromGregorian 2021 06 28) (Just $ fromGregorian 2021 07 02))
-            , (DateSpan (Just $ fromGregorian 2021 07 02) (Just $ fromGregorian 2021 07 05))
+      splitSpan False (DaysOfWeek [1, 5]) (DateSpan (Just $ Exact $ fromGregorian 2021 07 01) (Just $ Exact $ fromGregorian 2021 07 08))
+        @?= [ (DateSpan (Just $ Exact $ fromGregorian 2021 06 28) (Just $ Exact $ fromGregorian 2021 07 02))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 07 02) (Just $ Exact $ fromGregorian 2021 07 05))
             -- next week
-            , (DateSpan (Just $ fromGregorian 2021 07 05) (Just $ fromGregorian 2021 07 09))
+            , (DateSpan (Just $ Exact $ fromGregorian 2021 07 05) (Just $ Exact $ fromGregorian 2021 07 09))
             ]
 
   , testCase "match dayOfWeek" $ do
-      let dayofweek n s = splitspan (nthdayofweekcontaining n) (\w -> (if w == 0 then id else applyN (n-1) nextday . applyN (fromInteger w) nextweek)) 1 s
-          match ds day = splitSpan (DaysOfWeek [day]) ds @?= dayofweek day ds
+      let dayofweek n = splitspan (nthdayofweekcontaining n) (\w -> (if w == 0 then id else applyN (n-1) nextday . applyN (fromInteger w) nextweek)) 1
+          matchdow ds day = splitSpan False (DaysOfWeek [day]) ds @?= dayofweek day ds
           ys2021 = fromGregorian 2021 01 01
           ye2021 = fromGregorian 2021 12 31
           ys2022 = fromGregorian 2022 01 01
-      mapM_ (match (DateSpan (Just ys2021) (Just ye2021))) [1..7]
-      mapM_ (match (DateSpan (Just ys2021) (Just ys2022))) [1..7]
-      mapM_ (match (DateSpan (Just ye2021) (Just ys2022))) [1..7]
+      mapM_ (matchdow (DateSpan (Just $ Exact ys2021) (Just $ Exact ye2021))) [1..7]
+      mapM_ (matchdow (DateSpan (Just $ Exact ys2021) (Just $ Exact ys2022))) [1..7]
+      mapM_ (matchdow (DateSpan (Just $ Exact ye2021) (Just $ Exact ys2022))) [1..7]
 
-      mapM_ (match (DateSpan (Just ye2021) Nothing)) [1..7]
-      mapM_ (match (DateSpan (Just ys2022) Nothing)) [1..7]
+      mapM_ (matchdow (DateSpan (Just $ Exact ye2021) Nothing)) [1..7]
+      mapM_ (matchdow (DateSpan (Just $ Exact ys2022) Nothing)) [1..7]
 
-      mapM_ (match (DateSpan Nothing (Just ye2021))) [1..7]
-      mapM_ (match (DateSpan Nothing (Just ys2022))) [1..7]
+      mapM_ (matchdow (DateSpan Nothing (Just $ Exact ye2021))) [1..7]
+      mapM_ (matchdow (DateSpan Nothing (Just $ Exact ys2022))) [1..7]
 
   ]
