@@ -22,11 +22,13 @@ module Hledger.Data.Journal (
   addPeriodicTransaction,
   addTransaction,
   journalInferMarketPricesFromTransactions,
-  journalApplyCommodityStyles,
+  journalInferCommodityStyles,
+  journalStyleAmounts,
   commodityStylesFromAmounts,
   journalCommodityStyles,
+  journalCommodityStylesWith,
   journalToCost,
-  journalAddInferredEquityPostings,
+  journalInferEquityFromCosts,
   journalInferCostsFromEquity,
   journalMarkRedundantCosts,
   journalReverse,
@@ -84,6 +86,8 @@ module Hledger.Data.Journal (
   journalNextTransaction,
   journalPrevTransaction,
   journalPostings,
+  journalPostingAmounts,
+  showJournalAmountsDebug,
   journalTransactionsSimilarTo,
   -- * Account types
   journalAccountType,
@@ -92,6 +96,7 @@ module Hledger.Data.Journal (
   journalPostingsAddAccountTags,
   -- journalPrices,
   journalConversionAccount,
+  journalConversionAccounts,
   -- * Misc
   canonicalStyleFrom,
   nulljournal,
@@ -144,6 +149,7 @@ import System.FilePath (takeFileName)
 import Data.Ord (comparing)
 import Hledger.Data.Dates (nulldate)
 import Data.List (sort)
+-- import Data.Function ((&))
 
 
 -- | A parser of text that runs in some monad, keeping a Journal as state.
@@ -340,6 +346,14 @@ journalPrevTransaction j t = journalTransactionAt j (tindex t - 1)
 journalPostings :: Journal -> [Posting]
 journalPostings = concatMap tpostings . jtxns
 
+-- | All posting amounts from this journal, in order.
+journalPostingAmounts :: Journal -> [MixedAmount]
+journalPostingAmounts = map pamount . journalPostings
+
+-- | Show the journal amounts rendered, suitable for debug logging.
+showJournalAmountsDebug :: Journal -> String
+showJournalAmountsDebug = show.map showMixedAmountOneLine.journalPostingAmounts
+
 -- | Sorted unique commodity symbols declared by commodity directives in this journal.
 journalCommoditiesDeclared :: Journal -> [CommoditySymbol]
 journalCommoditiesDeclared = M.keys . jcommodities
@@ -506,13 +520,6 @@ letterPairs :: String -> [String]
 letterPairs (a:b:rest) = [a,b] : letterPairs (b:rest)
 letterPairs _ = []
 
--- | The 'AccountName' to use for automatically generated conversion postings.
-journalConversionAccount :: Journal -> AccountName
-journalConversionAccount =
-    headDef (T.pack "equity:conversion")
-    . M.findWithDefault [] Conversion
-    . jdeclaredaccounttypes
-
 -- Newer account type code.
 
 journalAccountType :: Journal -> AccountName -> Maybe AccountType
@@ -544,7 +551,7 @@ journalAccountTypes j = M.fromList [(a,acctType) | (a, Just (acctType,_)) <- fla
                             then mparenttype
                             else (,False) <$> accountNameInferType a <|> mparenttype
 
--- | Build a map of the account types explicitly declared.
+-- | Build a map of the account types explicitly declared for each account.
 journalDeclaredAccountTypes :: Journal -> M.Map AccountName AccountType
 journalDeclaredAccountTypes Journal{jdeclaredaccounttypes} =
   M.fromList $ concat [map (,t) as | (t,as) <- M.toList jdeclaredaccounttypes]
@@ -556,6 +563,19 @@ journalPostingsAddAccountTags :: Journal -> Journal
 journalPostingsAddAccountTags j = journalMapPostings addtags j
   where addtags p = p `postingAddTags` (journalInheritedAccountTags j $ paccount p)
 
+-- | The account to use for automatically generated conversion postings in this journal:
+-- the first of the journalConversionAccounts.
+journalConversionAccount :: Journal -> AccountName
+journalConversionAccount = headDef defaultConversionAccount . journalConversionAccounts
+
+-- | All the accounts declared or inferred as Conversion type in this journal.
+journalConversionAccounts :: Journal -> [AccountName]
+journalConversionAccounts = M.keys . M.filter (==Conversion) . jaccounttypes
+
+-- The fallback account to use for automatically generated conversion postings
+-- if no account is declared with the Conversion type.
+defaultConversionAccount = "equity:conversion"
+
 -- Various kinds of filtering on journals. We do it differently depending
 -- on the command.
 
@@ -792,17 +812,19 @@ journalModifyTransactions verbosetags d j =
     Right ts -> Right j{jtxns=ts}
     Left err -> Left err
 
--- | Choose and apply a consistent display style to the posting
--- amounts in each commodity (see journalCommodityStyles).
--- Can return an error message eg if inconsistent number formats are found.
-journalApplyCommodityStyles :: Journal -> Either String Journal
-journalApplyCommodityStyles = fmap fixjournal . journalInferCommodityStyles
+-- | Apply this journal's commodity display styles to all of its amounts.
+-- This does no display rounding, keeping decimal digits as they were;
+-- it is suitable for an early cleanup pass before calculations.
+-- Reports may want to do additional rounding/styling at render time.
+-- This can return an error message eg if inconsistent number formats are found.
+journalStyleAmounts :: Journal -> Either String Journal
+journalStyleAmounts = fmap journalapplystyles . journalInferCommodityStyles
   where
-    fixjournal j@Journal{jpricedirectives=pds} =
-        journalMapPostings (postingApplyCommodityStyles styles) j{jpricedirectives=map fixpricedirective pds}
+    journalapplystyles j@Journal{jpricedirectives=pds} =
+      journalMapPostings (styleAmounts styles) j{jpricedirectives=map fixpricedirective pds}
       where
-        styles = journalCommodityStyles j
-        fixpricedirective pd@PriceDirective{pdamount=a} = pd{pdamount=styleAmountExceptPrecision styles a}
+        styles = journalCommodityStylesWith NoRounding j  -- defer rounding, in case of print --round=none
+        fixpricedirective pd@PriceDirective{pdamount=a} = pd{pdamount=styleAmounts styles a}
 
 -- | Get the canonical amount styles for this journal, whether (in order of precedence):
 -- set globally in InputOpts,
@@ -821,6 +843,11 @@ journalCommodityStyles j =
     defaultcommoditystyle = M.fromList $ catMaybes [jparsedefaultcommodity j]
     inferredstyles        = jinferredcommodities j
 
+-- | Like journalCommodityStyles, but attach a particular rounding strategy to the styles,
+-- affecting how they will affect display precisions when applied.
+journalCommodityStylesWith :: Rounding -> Journal -> M.Map CommoditySymbol AmountStyle
+journalCommodityStylesWith r = amountStylesSetRounding r . journalCommodityStyles
+
 -- | Collect and save inferred amount styles for each commodity based on
 -- the posting amounts in that commodity (excluding price amounts), ie:
 -- "the format of the first amount, adjusted to the highest precision of all amounts".
@@ -858,7 +885,7 @@ canonicalStyleFrom = foldl' canonicalStyle amountstyle
 -- with the first digit group style seen,
 -- with the maximum precision of all.
 canonicalStyle :: AmountStyle -> AmountStyle -> AmountStyle
-canonicalStyle a b = a{asprecision=prec, asdecimalpoint=decmark, asdigitgroups=mgrps}
+canonicalStyle a b = a{asprecision=prec, asdecimalmark=decmark, asdigitgroups=mgrps}
   where
     -- precision is maximum of all precisions
     prec = max (asprecision a) (asprecision b)
@@ -874,7 +901,7 @@ canonicalStyle a b = a{asprecision=prec, asdecimalpoint=decmark, asdigitgroups=m
     -- urgh.. refactor..
     decmark = case mgrps of
         Just _  -> Just defdecmark
-        Nothing -> asdecimalpoint a <|> asdecimalpoint b <|> Just defdecmark
+        Nothing -> asdecimalmark a <|> asdecimalmark b <|> Just defdecmark
 
 -- -- | Apply this journal's historical price records to unpriced amounts where possible.
 -- journalApplyPriceDirectives :: Journal -> Journal
@@ -908,26 +935,26 @@ journalInferMarketPricesFromTransactions j =
        journalPostings j
    }
 
--- | Convert all this journal's amounts to cost using the transaction prices, if any.
--- The journal's commodity styles are applied to the resulting amounts.
+-- | Convert all this journal's amounts to cost using their attached prices, if any.
 journalToCost :: ConversionOp -> Journal -> Journal
-journalToCost cost j@Journal{jtxns=ts} = j{jtxns=map (transactionToCost styles cost) ts}
-  where
-    styles = journalCommodityStyles j
+journalToCost cost j@Journal{jtxns=ts} = j{jtxns=map (transactionToCost cost) ts}
 
--- | Add inferred equity postings to a 'Journal' using transaction prices.
-journalAddInferredEquityPostings :: Bool -> Journal -> Journal
-journalAddInferredEquityPostings verbosetags j = journalMapTransactions (transactionAddInferredEquityPostings verbosetags equityAcct) j
+-- | Add equity postings inferred from costs, where needed and possible.
+-- See hledger manual > Cost reporting.
+journalInferEquityFromCosts :: Bool -> Journal -> Journal
+journalInferEquityFromCosts verbosetags j = journalMapTransactions (transactionAddInferredEquityPostings verbosetags equityAcct) j
   where
     equityAcct = journalConversionAccount j
 
 -- | Add costs inferred from equity conversion postings, where needed and possible.
--- See hledger manual > Inferring cost from equity postings.
+-- See hledger manual > Cost reporting.
 journalInferCostsFromEquity :: Journal -> Either String Journal
 journalInferCostsFromEquity j = do
-    ts <- mapM (transactionInferCostsFromEquity False $ jaccounttypes j) $ jtxns j
-    return j{jtxns=ts}
+  ts <- mapM (transactionInferCostsFromEquity False conversionaccts) $ jtxns j
+  return j{jtxns=ts}
+  where conversionaccts = journalConversionAccounts j
 
+-- XXX duplication of the above
 -- | Do just the internal tagging that is normally done by journalInferCostsFromEquity,
 -- identifying equity conversion postings and, in particular, postings which have redundant costs.
 -- Tagging the latter is useful as it allows them to be ignored during transaction balancedness checking.
@@ -935,8 +962,9 @@ journalInferCostsFromEquity j = do
 -- when it will have more information (amounts) to work with.
 journalMarkRedundantCosts :: Journal -> Either String Journal
 journalMarkRedundantCosts j = do
-    ts <- mapM (transactionInferCostsFromEquity True $ jaccounttypes j) $ jtxns j
-    return j{jtxns=ts}
+  ts <- mapM (transactionInferCostsFromEquity True conversionaccts) $ jtxns j
+  return j{jtxns=ts}
+  where conversionaccts = journalConversionAccounts j
 
 -- -- | Get this journal's unique, display-preference-canonicalised commodities, by symbol.
 -- journalCanonicalCommodities :: Journal -> M.Map String CommoditySymbol
@@ -1083,16 +1111,24 @@ transactionPivot fieldortagname t = t{tpostings = map (postingPivot fieldortagna
 -- | Replace this posting's account name with the value
 -- of the given field or tag, if any, otherwise the empty string.
 postingPivot :: Text -> Posting -> Posting
-postingPivot fieldortagname p = p{paccount = pivotedacct, poriginal = Just $ originalPosting p}
-  where
-    pivotedacct
-      | Just t <- ptransaction p, fieldortagname == "code"        = tcode t
-      | Just t <- ptransaction p, fieldortagname == "description" = tdescription t
-      | Just t <- ptransaction p, fieldortagname == "payee"       = transactionPayee t
-      | Just t <- ptransaction p, fieldortagname == "note"        = transactionNote t
-      | Just t <- ptransaction p, fieldortagname == "status"      = T.pack . show . tstatus $ t
-      | Just (_, value) <- postingFindTag fieldortagname p        = value
-      | otherwise                                                 = ""
+postingPivot fieldortagname p =
+  p{paccount = pivotAccount fieldortagname p, poriginal = Just $ originalPosting p}
+
+pivotAccount :: Text -> Posting -> Text
+pivotAccount fieldortagname p =
+  T.intercalate ":" [pivotComponent x p | x <- T.splitOn ":" fieldortagname]
+
+pivotComponent :: Text -> Posting -> Text
+pivotComponent fieldortagname p
+  |                           fieldortagname == "acct"        = paccount p
+  | Just t <- ptransaction p, fieldortagname == "code"        = tcode t
+  | Just t <- ptransaction p, fieldortagname == "desc"        = tdescription t
+  | Just t <- ptransaction p, fieldortagname == "description" = tdescription t  -- backward compatible with 1.30 and older
+  | Just t <- ptransaction p, fieldortagname == "payee"       = transactionPayee t
+  | Just t <- ptransaction p, fieldortagname == "note"        = transactionNote t
+  | Just t <- ptransaction p, fieldortagname == "status"      = T.pack . show . tstatus $ t
+  | Just (_, value) <- postingFindTag fieldortagname p        = value
+  | otherwise                                                 = ""
 
 postingFindTag :: TagName -> Posting -> Maybe (TagName, TagValue)
 postingFindTag tagname p = find ((tagname==) . fst) $ postingAllTags p
