@@ -2,6 +2,7 @@
 JSON instances. Should they be in Types.hs ?
 -}
 
+{-# LANGUAGE CPP               #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE LambdaCase        #-}
 {-# LANGUAGE OverloadedStrings #-}
@@ -67,7 +68,13 @@ instance (Integral a, ToJSON a) => ToJSON (DecimalRaw a) where
   toJSON = object . decimalKV
   toEncoding = pairs . mconcat . decimalKV
 
-decimalKV :: (KeyValue kv, Integral a, ToJSON a) => DecimalRaw a -> [kv]
+decimalKV :: (
+#if MIN_VERSION_aeson(2,2,0)
+  KeyValue e kv,
+#else
+  KeyValue kv,
+#endif
+  Integral a, ToJSON a) => DecimalRaw a -> [kv]
 decimalKV d = let d' = if decimalPlaces d <= 10 then d else roundTo 10 d in
     [ "decimalPlaces"   .= decimalPlaces d'
     , "decimalMantissa" .= decimalMantissa d'
@@ -75,6 +82,7 @@ decimalKV d = let d' = if decimalPlaces d <= 10 then d else roundTo 10 d in
     ]
 
 instance ToJSON Amount
+instance ToJSON Rounding
 instance ToJSON AmountStyle
 
 -- Use the same JSON serialisation as Maybe Word8
@@ -102,7 +110,13 @@ instance ToJSON Posting where
   toJSON = object . postingKV
   toEncoding = pairs . mconcat . postingKV
 
-postingKV :: KeyValue kv => Posting -> [kv]
+postingKV ::
+#if MIN_VERSION_aeson(2,2,0)
+  KeyValue e kv
+#else
+  KeyValue kv
+#endif
+  => Posting -> [kv]
 postingKV Posting{..} =
     [ "pdate"             .= pdate
     , "pdate2"            .= pdate2
@@ -144,22 +158,29 @@ instance ToJSON Account where
   toJSON = object . accountKV
   toEncoding = pairs . mconcat . accountKV
 
-accountKV :: KeyValue kv => Account -> [kv]
+accountKV ::
+#if MIN_VERSION_aeson(2,2,0)
+  KeyValue e kv
+#else
+  KeyValue kv
+#endif
+  => Account -> [kv]
 accountKV a =
-    [ "aname"        .= aname a
-    , "aebalance"    .= aebalance a
-    , "aibalance"    .= aibalance a
-    , "anumpostings" .= anumpostings a
-    , "aboring"      .= aboring a
+    [ "aname"            .= aname a
+    , "adeclarationinfo" .= adeclarationinfo a
+    , "aebalance"        .= aebalance a
+    , "aibalance"        .= aibalance a
+    , "anumpostings"     .= anumpostings a
+    , "aboring"          .= aboring a
     -- To avoid a cycle, show just the parent account's name
     -- in a dummy field. When re-parsed, there will be no parent.
-    , "aparent_"     .= maybe "" aname (aparent a)
+    , "aparent_"         .= maybe "" aname (aparent a)
     -- Just the names of subaccounts, as a dummy field, ignored when parsed.
-    , "asubs_"       .= map aname (asubs a)
+    , "asubs_"           .= map aname (asubs a)
     -- The actual subaccounts (and their subs..), making a (probably highly redundant) tree
     -- ,"asubs"        .= asubs a
     -- Omit the actual subaccounts
-    , "asubs"        .= ([]::[Account])
+    , "asubs"            .= ([]::[Account])
     ]
 
 instance ToJSON Ledger
@@ -173,6 +194,7 @@ instance FromJSON Pos where
   parseJSON = fmap mkPos . parseJSON
 
 instance FromJSON Amount
+instance FromJSON Rounding
 instance FromJSON AmountStyle
 
 -- Use the same JSON serialisation as Maybe Word8
