@@ -39,6 +39,7 @@ module Hledger.Data.Posting (
   postingStripPrices,
   postingApplyAliases,
   postingApplyCommodityStyles,
+  postingStyleAmounts,
   postingAddTags,
   -- * date operations
   postingDate,
@@ -51,6 +52,7 @@ module Hledger.Data.Posting (
   -- * comment/tag operations
   commentJoin,
   commentAddTag,
+  commentAddTagUnspaced,
   commentAddTagNextLine,
   -- * arithmetic
   sumPostings,
@@ -59,7 +61,10 @@ module Hledger.Data.Posting (
   showPostingLines,
   postingAsLines,
   postingsAsLines,
+  postingsAsLinesBeancount,
+  postingAsLinesBeancount,
   showAccountName,
+  showAccountNameBeancount,
   renderCommentLines,
   showBalanceAssertion,
   -- * misc.
@@ -97,6 +102,27 @@ import Hledger.Data.Dates (nulldate, spanContainsDate)
 import Hledger.Data.Valuation
 
 
+instance HasAmounts BalanceAssertion where
+  styleAmounts styles ba@BalanceAssertion{baamount} = ba{baamount=styleAmounts styles baamount}
+
+instance HasAmounts Posting where
+  styleAmounts styles p@Posting{pamount, pbalanceassertion} =
+    p{ pamount=styleAmounts styles pamount
+      ,pbalanceassertion=styleAmounts styles pbalanceassertion 
+      }
+
+{-# DEPRECATED postingApplyCommodityStyles "please use styleAmounts instead" #-}
+-- | Find and apply the appropriate display style to the posting amounts
+-- in each commodity (see journalCommodityStyles).
+-- Main amount precisions may be set or not according to the styles, but cost precisions are not set.
+postingApplyCommodityStyles :: M.Map CommoditySymbol AmountStyle -> Posting -> Posting
+postingApplyCommodityStyles = styleAmounts
+
+{-# DEPRECATED postingStyleAmounts "please use styleAmounts instead" #-}
+-- | Like postingApplyCommodityStyles, but neither
+-- main amount precisions or cost precisions are set.
+postingStyleAmounts :: M.Map CommoditySymbol AmountStyle -> Posting -> Posting
+postingStyleAmounts = styleAmounts
 
 nullposting, posting :: Posting
 nullposting = Posting
@@ -220,6 +246,9 @@ postingsAsLines onelineamounts ps = concatMap first3 linesWithWidths
 -- Or if onelineamounts is true, such amounts are shown on one line, comma-separated
 -- (and the output will not be valid journal syntax).
 --
+-- If an amount is zero, any commodity symbol attached to it is shown
+-- (and the corresponding commodity display style is used).
+--
 -- By default, 4 spaces (2 if there's a status flag) are shown between
 -- account name and start of amount area, which is typically 12 chars wide
 -- and contains a right-aligned amount (so 10-12 visible spaces between
@@ -262,7 +291,10 @@ postingAsLines elideamount onelineamounts acctwidth amtwidth p =
     -- amtwidth at all.
     shownAmounts
       | elideamount = [mempty]
-      | otherwise   = showMixedAmountLinesB noColour{displayOneLine=onelineamounts} $ pamount p
+      | otherwise   = showMixedAmountLinesB displayopts $ pamount p
+        where displayopts = noColour{
+          displayZeroCommodity=True, displayForceDecimalMark=True, displayOneLine=onelineamounts
+          }
     thisamtwidth = maximumBound 0 $ map wbWidth shownAmounts
 
     -- when there is a balance assertion, show it only on the last posting line
@@ -289,6 +321,91 @@ showAccountName w = fmt
     fmt VirtualPosting         = wrap "(" ")" . maybe id (T.takeEnd . subtract 2) w
     fmt BalancedVirtualPosting = wrap "[" "]" . maybe id (T.takeEnd . subtract 2) w
 
+-- | Like postingsAsLines but generates Beancount journal format.
+postingsAsLinesBeancount :: [Posting] -> [Text]
+postingsAsLinesBeancount ps = concatMap first3 linesWithWidths
+  where
+    linesWithWidths = map (postingAsLinesBeancount False maxacctwidth maxamtwidth) ps
+    maxacctwidth = maximumBound 0 $ map second3 linesWithWidths
+    maxamtwidth  = maximumBound 0 $ map third3  linesWithWidths
+
+-- | Like postingAsLines but generates Beancount journal format.
+postingAsLinesBeancount  :: Bool -> Int -> Int -> Posting -> ([Text], Int, Int)
+postingAsLinesBeancount elideamount acctwidth amtwidth p =
+    (concatMap (++ newlinecomments) postingblocks, thisacctwidth, thisamtwidth)
+  where
+    -- This needs to be converted to strict Text in order to strip trailing
+    -- spaces. This adds a small amount of inefficiency, and the only difference
+    -- is whether there are trailing spaces in print (and related) reports. This
+    -- could be removed and we could just keep everything as a Text Builder, but
+    -- would require adding trailing spaces to 42 failing tests.
+    postingblocks = [map T.stripEnd . T.lines . TL.toStrict $
+                       render [ textCell BottomLeft statusandaccount
+                              , textCell BottomLeft "  "
+                              , Cell BottomLeft [pad amt]
+                              , textCell BottomLeft samelinecomment
+                              ]
+                    | (amt,_assertion) <- shownAmountsAssertions]
+    render = renderRow def{tableBorders=False, borderSpaces=False} . Group NoLine . map Header
+    pad amt = WideBuilder (TB.fromText $ T.replicate w " ") w <> amt
+      where w = max 12 amtwidth - wbWidth amt  -- min. 12 for backwards compatibility
+
+    pacct = showAccountNameBeancount Nothing $ paccount p
+    pstatusandacct p' = if pstatus p' == Pending then "! " else "" <> pacct
+
+    -- currently prices are considered part of the amount string when right-aligning amounts
+    -- Since we will usually be calling this function with the knot tied between
+    -- amtwidth and thisamtwidth, make sure thisamtwidth does not depend on
+    -- amtwidth at all.
+    shownAmounts
+      | elideamount = [mempty]
+      | otherwise   = showMixedAmountLinesB displayopts a'
+        where
+          displayopts = noColour{ displayZeroCommodity=True, displayForceDecimalMark=True }
+          a' = mapMixedAmount amountToBeancount $ pamount p
+    thisamtwidth = maximumBound 0 $ map wbWidth shownAmounts
+
+    -- when there is a balance assertion, show it only on the last posting line
+    shownAmountsAssertions = zip shownAmounts shownAssertions
+      where
+        shownAssertions = replicate (length shownAmounts - 1) mempty ++ [assertion]
+          where
+            assertion = maybe mempty ((WideBuilder (TB.singleton ' ') 1 <>).showBalanceAssertion) $ pbalanceassertion p
+
+    -- pad to the maximum account name width, plus 2 to leave room for status flags, to keep amounts aligned
+    statusandaccount = lineIndent . fitText (Just $ 2 + acctwidth) Nothing False True $ pstatusandacct p
+    thisacctwidth = realLength pacct
+
+    (samelinecomment, newlinecomments) =
+      case renderCommentLines (pcomment p) of []   -> ("",[])
+                                              c:cs -> (c,cs)
+
+type BeancountAmount = Amount
+
+-- | Do some best effort adjustments to make an amount that renders
+-- in a way that Beancount can read: forces the commodity symbol to the right,
+-- converts a few currency symbols to names, capitalises all letters.
+amountToBeancount :: Amount -> BeancountAmount
+amountToBeancount a@Amount{acommodity=c,astyle=s,aprice=mp} = a{acommodity=c', astyle=s', aprice=mp'}
+  -- https://beancount.github.io/docs/beancount_language_syntax.html#commodities-currencies
+  where
+    c' = T.toUpper $
+      T.replace "$" "USD" $
+      T.replace "€" "EUR" $
+      T.replace "¥" "JPY" $
+      T.replace "£" "GBP" $
+      c
+    s' = s{ascommodityside=R, ascommodityspaced=True}
+    mp' = costToBeancount <$> mp
+      where
+        costToBeancount (TotalPrice amt) = TotalPrice $ amountToBeancount amt
+        costToBeancount (UnitPrice  amt) = UnitPrice  $ amountToBeancount amt
+
+-- | Like showAccountName for Beancount journal format.
+-- Calls accountNameToBeancount first.
+showAccountNameBeancount :: Maybe Int -> AccountName -> Text
+showAccountNameBeancount w = maybe id T.take w . accountNameToBeancount
+
 -- | Render a transaction or posting's comment as indented, semicolon-prefixed comment lines.
 -- The first line (unless empty) will have leading space, subsequent lines will have a larger indent.
 renderCommentLines :: Text -> [Text]
@@ -407,14 +524,6 @@ postingApplyAliases aliases p@Posting{paccount} =
         err = "problem while applying account aliases:\n" ++ pshow aliases
           ++ "\n to account name: "++T.unpack paccount++"\n "++e
 
--- | Choose and apply a consistent display style to the posting
--- amounts in each commodity (see journalCommodityStyles).
-postingApplyCommodityStyles :: M.Map CommoditySymbol AmountStyle -> Posting -> Posting
-postingApplyCommodityStyles styles p = p{pamount=styleMixedAmount styles $ pamount p
-                                        ,pbalanceassertion=fixbalanceassertion <$> pbalanceassertion p}
-  where
-    fixbalanceassertion ba = ba{baamount=styleAmountExceptPrecision styles $ baamount ba}
-
 -- | Add tags to a posting, discarding any for which the posting already has a value.
 postingAddTags :: Posting -> [Tag] -> Posting
 postingAddTags p@Posting{ptags} tags = p{ptags=ptags `union` tags}
@@ -426,16 +535,15 @@ postingApplyValuation :: PriceOracle -> M.Map CommoditySymbol AmountStyle -> Day
 postingApplyValuation priceoracle styles periodlast today v p =
     postingTransformAmount (mixedAmountApplyValuation priceoracle styles periodlast today (postingDate p) v) p
 
--- | Maybe convert this 'Posting's amount to cost, and apply apply appropriate
--- amount styles.
-postingToCost :: M.Map CommoditySymbol AmountStyle -> ConversionOp -> Posting -> Maybe Posting
-postingToCost _      NoConversionOp p = Just p
-postingToCost styles ToCost         p
-    -- If this is a conversion posting with a matched transaction price posting, ignore it
-    | "_conversion-matched" `elem` map fst (ptags p) && noCost = Nothing
-    | otherwise = Just $ postingTransformAmount (styleMixedAmount styles . mixedAmountCost) p
+-- | Maybe convert this 'Posting's amount to cost.
+postingToCost :: ConversionOp -> Posting -> Maybe Posting
+postingToCost NoConversionOp p = Just p
+postingToCost ToCost         p
+  -- If this is a conversion posting with a matched transaction price posting, ignore it
+  | "_conversion-matched" `elem` map fst (ptags p) && nocosts = Nothing
+  | otherwise = Just $ postingTransformAmount mixedAmountCost p
   where
-    noCost = (not . any (isJust . aprice) . amountsRaw) $ pamount p
+    nocosts = (not . any (isJust . aprice) . amountsRaw) $ pamount p
 
 -- | Generate inferred equity postings from a 'Posting' using transaction prices.
 -- Make sure not to generate equity postings when there are already matched
@@ -451,7 +559,7 @@ postingAddInferredEquityPostings verbosetags equityAcct p
     conversionPostings amt = case aprice amt of
         Nothing -> []
         Just _  -> [ cp{ paccount = accountPrefix <> amtCommodity
-                       , pamount = mixedAmount . negate $ amountStripPrices amt
+                       , pamount = mixedAmount . negate $ amountStripCost amt
                        }
                    , cp{ paccount = accountPrefix <> costCommodity
                        , pamount = mixedAmount cost
@@ -504,6 +612,15 @@ commentAddTag c (t,v)
     c'  = T.stripEnd c
     tag = t <> ": " <> v
 
+-- | Like commentAddTag, but omits the space after the colon.
+commentAddTagUnspaced :: Text -> Tag -> Text
+commentAddTagUnspaced c (t,v)
+  | T.null c' = tag
+  | otherwise = c' `commentJoin` tag
+  where
+    c'  = T.stripEnd c
+    tag = t <> ":" <> v
+
 -- | Add a tag on its own line to a comment, preserving any prior content.
 -- A space is inserted following the colon, before the value.
 commentAddTagNextLine :: Text -> Tag -> Text
