@@ -32,6 +32,7 @@ module Hledger.Data.Transaction
 , transactionApplyAliases
 , transactionMapPostings
 , transactionMapPostingAmounts
+, transactionAmounts
 , partitionAndCheckConversionPostings
   -- nonzerobalanceerror
   -- * date operations
@@ -45,6 +46,7 @@ module Hledger.Data.Transaction
 , showTransaction
 , showTransactionOneLineAmounts
 , showTransactionLineFirstPart
+, showTransactionBeancount
 , transactionFile
   -- * transaction errors
 , annotateErrorWithTransaction
@@ -53,8 +55,8 @@ module Hledger.Data.Transaction
 ) where
 
 import Control.Monad.Trans.State (StateT(..), evalStateT)
-import Data.Bifunctor (first)
-import Data.Foldable (foldrM)
+import Data.Bifunctor (first, second)
+import Data.Foldable (foldlM)
 import Data.Maybe (fromMaybe, isJust, mapMaybe)
 import Data.Semigroup (Endo(..))
 import Data.Text (Text)
@@ -70,8 +72,13 @@ import Hledger.Data.Dates
 import Hledger.Data.Posting
 import Hledger.Data.Amount
 import Hledger.Data.Valuation
+import Data.Decimal (normalizeDecimal, decimalPlaces)
+import Data.Functor ((<&>))
 
 
+instance HasAmounts Transaction where
+  styleAmounts styles t = t{tpostings=styleAmounts styles $ tpostings t}
+
 nulltransaction :: Transaction
 nulltransaction = Transaction {
                     tindex=0,
@@ -108,6 +115,12 @@ payeeAndNoteFromDescription t
   where
     (p, n) = T.span (/= '|') t
 
+-- | Like payeeAndNoteFromDescription, but if there's no | then payee is empty.
+payeeAndNoteFromDescription' :: Text -> (Text,Text)
+payeeAndNoteFromDescription' t =
+  if isJust $ T.find (=='|') t then payeeAndNoteFromDescription t else ("",t)
+
+
 {-|
 Render a journal transaction as text similar to the style of Ledger's print command.
 
@@ -163,6 +176,33 @@ showTransactionLineFirstPart t = T.concat [date, status, code]
            | otherwise            = ""
     code = if T.null (tcode t) then "" else wrap " (" ")" $ tcode t
 
+-- | Like showTransaction, but generates Beancount journal format.
+showTransactionBeancount :: Transaction -> Text
+showTransactionBeancount t =
+  -- https://beancount.github.io/docs/beancount_language_syntax.html
+  -- similar to showTransactionHelper, but I haven't bothered with Builder
+     firstline <> nl
+  <> foldMap ((<> nl)) newlinecomments
+  <> foldMap ((<> nl)) (postingsAsLinesBeancount $ tpostings t)
+  <> nl
+  where
+    nl = "\n"
+    firstline = T.concat [date, status, payee, note, tags, samelinecomment]
+    date = showDate $ tdate t
+    status = if tstatus t == Pending then " !" else " *"
+    (payee,note) =
+      case payeeAndNoteFromDescription' $ tdescription t of
+        ("","") -> ("",      ""      )
+        ("",n ) -> (""     , wrapq n )
+        (p ,"") -> (wrapq p, wrapq "")
+        (p ,n ) -> (wrapq p, wrapq n )
+      where
+        wrapq = wrap " \"" "\"" . escapeDoubleQuotes . escapeBackslash
+    tags = T.concat $ map ((" #"<>).fst) $ ttags t
+    (samelinecomment, newlinecomments) =
+      case renderCommentLines (tcomment t) of []   -> ("",[])
+                                              c:cs -> (c,cs)
+
 hasRealPostings :: Transaction -> Bool
 hasRealPostings = not . null . realPostings
 
@@ -216,10 +256,9 @@ transactionApplyValuation :: PriceOracle -> M.Map CommoditySymbol AmountStyle ->
 transactionApplyValuation priceoracle styles periodlast today v =
   transactionTransformPostings (postingApplyValuation priceoracle styles periodlast today v)
 
--- | Maybe convert this 'Transaction's amounts to cost and apply the
--- appropriate amount styles.
-transactionToCost :: M.Map CommoditySymbol AmountStyle -> ConversionOp -> Transaction -> Transaction
-transactionToCost styles cost t = t{tpostings = mapMaybe (postingToCost styles cost) $ tpostings t}
+-- | Maybe convert this 'Transaction's amounts to cost.
+transactionToCost :: ConversionOp -> Transaction -> Transaction
+transactionToCost cost t = t{tpostings = mapMaybe (postingToCost cost) $ tpostings t}
 
 -- | Add inferred equity postings to a 'Transaction' using transaction prices.
 transactionAddInferredEquityPostings :: Bool -> AccountName -> Transaction -> Transaction
@@ -228,9 +267,14 @@ transactionAddInferredEquityPostings verbosetags equityAcct t =
 
 type IdxPosting = (Int, Posting)
 
--- WARNING: twisty code ahead
+-- XXX Warning: The following code - for analysing equity conversion postings,
+-- inferring missing costs and ignoring redundant costs -
+-- is twisty and hard to follow.
+
+label s = ((s <> ": ")++)
 
 -- | Add costs inferred from equity postings in this transaction.
+-- The name(s) of conversion equity accounts should be provided.
 -- For every adjacent pair of conversion postings, it will first search the postings
 -- with costs to see if any match. If so, it will tag these as matched.
 -- If no postings with costs match, it will then search the postings without costs,
@@ -238,73 +282,119 @@ type IdxPosting = (Int, Posting)
 -- If it finds a match, it will add a cost and then tag it.
 -- If the first argument is true, do a dry run instead: identify and tag
 -- the costful and conversion postings, but don't add costs.
-transactionInferCostsFromEquity :: Bool -> M.Map AccountName AccountType -> Transaction -> Either String Transaction
-transactionInferCostsFromEquity dryrun acctTypes t = first (annotateErrorWithTransaction t . T.unpack) $ do
-    (conversionPairs, stateps) <- partitionAndCheckConversionPostings False acctTypes npostings
-    f <- transformIndexedPostingsF (addCostsToPostings dryrun) conversionPairs stateps
-    return t{tpostings = map (snd . f) npostings}
+transactionInferCostsFromEquity :: Bool -> [AccountName] -> Transaction -> Either String Transaction
+transactionInferCostsFromEquity dryrun conversionaccts t = first (annotateErrorWithTransaction t . T.unpack) $ do
+  -- number the postings
+  let npostings = zip [0..] $ tpostings t
+
+  -- Identify all pairs of conversion postings and all other postings (with and without costs) in the transaction.
+  (conversionPairs, otherps) <- partitionAndCheckConversionPostings False conversionaccts npostings
+
+  -- Generate a pure function that can be applied to each of this transaction's postings,
+  -- possibly modifying it, to produce the following end result:
+  -- 1. each pair of conversion postings, and the corresponding postings which balance them, are tagged for easy identification
+  -- 2. each pair of balancing postings which did't have an explicit cost, have had a cost calculated and added to one of them
+  -- 3. if any ambiguous situation was detected, an informative error is raised
+  processposting <- transformIndexedPostingsF (addCostsToPostings dryrun) conversionPairs otherps
+
+  -- And if there was no error, use it to modify the transaction's postings.
+  return t{tpostings = map (snd . processposting) npostings}
+
   where
-    -- Include indices for postings
-    npostings = zip [0..] $ tpostings t
-    transformIndexedPostingsF f = evalStateT . fmap (appEndo . foldMap Endo) . traverse f
 
-    -- Given a pair of indexed conversion postings, and a state consisting of lists of
-    -- costful and costless non-conversion postings, create a function which adds a conversion cost
-    -- to the posting which matches the conversion postings if necessary,
-    -- and tags the conversion and matched postings. Then update the state by removing the
-    -- matched postings. If there are no matching postings or too much ambiguity,
-    -- return an error string annotated with the conversion postings.
-    -- If the first argument is true, do a dry run instead: identify and tag
-    -- the costful and conversion postings, but don't add costs.
-    addCostsToPostings :: Bool -> (IdxPosting, IdxPosting)
-                        -> StateT ([IdxPosting], [IdxPosting]) (Either Text) (IdxPosting -> IdxPosting)
+    -- Generate the tricksy processposting function,
+    -- which when applied to each posting in turn, rather magically has the effect of
+    -- applying addCostsToPostings to each pair of conversion postings in the transaction,
+    -- matching them with the other postings, tagging them and perhaps adding cost information to the other postings.
+    -- General type:
+    -- transformIndexedPostingsF :: (Monad m, Foldable t, Traversable t) =>
+    --   (a -> StateT s m (a1 -> a1)) ->
+    --   t a ->
+    --   s ->
+    --   m (a1 -> a1)
+    -- Concrete type:
+    transformIndexedPostingsF ::
+      ((IdxPosting, IdxPosting) -> StateT ([IdxPosting],[IdxPosting]) (Either Text) (IdxPosting -> IdxPosting)) ->  -- state update function (addCostsToPostings with the bool applied)
+      [(IdxPosting, IdxPosting)] ->   -- initial state: the pairs of adjacent conversion postings in the transaction
+      ([IdxPosting],[IdxPosting]) ->  -- initial state: the other postings in the transaction, separated into costful and costless
+      (Either Text (IdxPosting -> IdxPosting))  -- returns an error message or a posting transform function
+    transformIndexedPostingsF updatefn = evalStateT . fmap (appEndo . foldMap Endo) . traverse (updatefn)
+
+    -- A tricksy state update helper for processposting/transformIndexedPostingsF.
+    -- Approximately: given a pair of conversion postings to match,
+    -- and lists of the remaining unmatched costful and costless other postings,
+    -- 1. find (and consume) two other postings which match the two conversion postings
+    -- 2. add identifying tags to the four postings
+    -- 3. add an explicit cost, if missing, to one of the matched other postings
+    -- 4. or if there is a problem, raise an informative error or do nothing as appropriate.
+    -- Or, if the first argument is true:
+    -- do a dry run instead: find and consume, add tags, but don't add costs
+    -- (and if there are no costful postings at all, do nothing).
+    addCostsToPostings :: Bool -> (IdxPosting, IdxPosting) -> StateT ([IdxPosting], [IdxPosting]) (Either Text) (IdxPosting -> IdxPosting)
     addCostsToPostings dryrun' ((n1, cp1), (n2, cp2)) = StateT $ \(costps, otherps) -> do
-        -- Get the two conversion posting amounts, if possible
-        ca1 <- conversionPostingAmountNoCost cp1
-        ca2 <- conversionPostingAmountNoCost cp2
-        let -- The function to add costs and tag postings in the indexed list of postings
-            transformPostingF np costp (n,p) =
-              (n, if | n == np            -> costp `postingAddTags` [("_price-matched","")]
-                     | n == n1 || n == n2 -> p     `postingAddTags` [("_conversion-matched","")]
-                     | otherwise          -> p)
-            -- All costful postings which match the conversion posting pair
-            matchingCostPs = mapMaybe (mapM $ costfulPostingIfMatchesBothAmounts ca1 ca2) costps
-            -- All other postings which match at least one of the conversion posting pair.
-            -- Add a corresponding cost to these postings, unless in dry run mode.
-            matchingOtherPs
-              | dryrun'   = [(n,(p, a)) | (n,p) <- otherps, let Just a = postingSingleAmount p]
-              | otherwise = mapMaybe (mapM $ addCostIfMatchesOneAmount ca1 ca2) otherps
+      -- Get the two conversion posting amounts, if possible
+      ca1 <- conversionPostingAmountNoCost cp1
+      ca2 <- conversionPostingAmountNoCost cp2
+      let 
+        -- All costful postings which match the conversion posting pair
+        matchingCostPs =
+          dbg7With (label "matched costful postings".show.length) $ 
+          mapMaybe (mapM $ costfulPostingIfMatchesBothAmounts ca1 ca2) costps
 
-        -- Annotate any errors with the conversion posting pair
-        first (annotateWithPostings [cp1, cp2]) $
-            if -- If a single costful posting matches the conversion postings,
-               -- delete it from the list of costful postings in the state, delete the
-               -- first matching costless posting from the list of costless postings
-               -- in the state, and return the transformation function with the new state.
-               | [(np, (costp, _))] <- matchingCostPs
-               , Just newcostps <- deleteIdx np costps
-                   -> Right (transformPostingF np costp, (if dryrun' then costps else newcostps, otherps))
-               -- If no costful postings match the conversion postings, but some
-               -- of the costless postings match, check that the first such posting has a
-               -- different amount from all the others, and if so add a cost to it,
-               -- then delete it from the list of costless postings in the state,
-               -- and return the transformation function with the new state.
-               | [] <- matchingCostPs
-               , (np, (costp, amt)):nps <- matchingOtherPs
-               , not $ any (amountMatches amt . snd . snd) nps
-               , Just newotherps <- deleteIdx np otherps
-                   -> Right (transformPostingF np costp, (costps, if dryrun' then otherps else newotherps))
-               -- Otherwise it's too ambiguous to make a guess, so return an error.
-               | otherwise -> Left "There is not a unique posting which matches the conversion posting pair:"
+        -- All other single-commodity postings whose amount matches at least one of the conversion postings,
+        -- with an explicit cost added. Or in dry run mode, all other single-commodity postings.
+        matchingOtherPs =
+          dbg7With (label "matched costless postings".show.length) $
+          if dryrun'
+          then [(n,(p, a)) | (n,p) <- otherps, let Just a = postingSingleAmount p]
+          else mapMaybe (mapM $ addCostIfMatchesOneAmount ca1 ca2) otherps
+
+        -- A function that adds a cost and/or tag to a numbered posting if appropriate.
+        postingAddCostAndOrTag np costp (n,p) =
+          (n, if | n == np            -> costp `postingAddTags` [("_price-matched","")]
+                 | n == n1 || n == n2 -> p     `postingAddTags` [("_conversion-matched","")]
+                 | otherwise          -> p)
+
+      -- Annotate any errors with the conversion posting pair
+      first (annotateWithPostings [cp1, cp2]) $
+        if
+          -- If a single costful posting matches the conversion postings,
+          -- delete it from the list of costful postings in the state, delete the
+          -- first matching costless posting from the list of costless postings
+          -- in the state, and return the transformation function with the new state.
+          | [(np, costp)] <- matchingCostPs
+          , Just newcostps <- deleteIdx np costps
+              -> Right (postingAddCostAndOrTag np costp, (if dryrun' then costps else newcostps, otherps))
+
+          -- If no costful postings match the conversion postings, but some
+          -- of the costless postings match, check that the first such posting has a
+          -- different amount from all the others, and if so add a cost to it,
+          -- then delete it from the list of costless postings in the state,
+          -- and return the transformation function with the new state.
+          | [] <- matchingCostPs
+          , (np, (costp, amt)):nps <- matchingOtherPs
+          , not $ any (amountsMatch amt . snd . snd) nps
+          , Just newotherps <- deleteIdx np otherps
+              -> Right (postingAddCostAndOrTag np costp, (costps, if dryrun' then otherps else newotherps))
+
+          -- Otherwise, do nothing, leaving the transaction unchanged.
+          -- We don't want to be over-zealous reporting problems here
+          -- since this is always called at least in dry run mode by
+          -- journalFinalise > journalMarkRedundantCosts. (#2045)
+          | otherwise -> Right (id, (costps, otherps))
 
     -- If a posting with cost matches both the conversion amounts, return it along
     -- with the matching amount which must be present in another non-conversion posting.
-    costfulPostingIfMatchesBothAmounts :: Amount -> Amount -> Posting -> Maybe (Posting, Amount)
-    costfulPostingIfMatchesBothAmounts a1 a2 p = do
-        a@Amount{aprice=Just _} <- postingSingleAmount p
-        if | amountMatches (-a1) a && amountMatches a2 (amountCost a) -> Just (p, -a2)
-           | amountMatches (-a2) a && amountMatches a1 (amountCost a) -> Just (p, -a1)
+    costfulPostingIfMatchesBothAmounts :: Amount -> Amount -> Posting -> Maybe Posting
+    costfulPostingIfMatchesBothAmounts a1 a2 costfulp = do
+        a@Amount{aprice=Just _} <- postingSingleAmount costfulp
+        if
+           | dbgamtmatch 1 a1 a (amountsMatch (-a1) a)  &&  dbgcostmatch 2 a2 a (amountsMatch a2 (amountCost a)) -> Just costfulp
+           | dbgamtmatch 2 a2 a (amountsMatch (-a2) a)  &&  dbgcostmatch 1 a1 a (amountsMatch a1 (amountCost a)) -> Just costfulp
            | otherwise -> Nothing
+           where
+            dbgamtmatch  n a b = dbg7 ("conversion posting "     <>show n<>" "<>showAmount a<>" balances amount "<>showAmountWithoutPrice b <>" of costful posting "<>showAmount b<>" at precision "<>dbgShowAmountPrecision a<>" ?")
+            dbgcostmatch n a b = dbg7 ("and\nconversion posting "<>show n<>" "<>showAmount a<>" matches cost "   <>showAmount (amountCost b)<>" of costful posting "<>showAmount b<>" at precision "<>dbgShowAmountPrecision a<>" ?") 
 
     -- Add a cost to a posting if it matches (negative) one of the
     -- supplied conversion amounts, adding the other amount as the cost.
@@ -312,9 +402,10 @@ transactionInferCostsFromEquity dryrun acctTypes t = first (annotateErrorWithTra
     addCostIfMatchesOneAmount a1 a2 p = do
         a <- postingSingleAmount p
         let newp cost = p{pamount = mixedAmount a{aprice = Just $ TotalPrice cost}}
-        if | amountMatches (-a1) a -> Just (newp a2, a2)
-           | amountMatches (-a2) a -> Just (newp a1, a1)
-           | otherwise             -> Nothing
+        if
+           | amountsMatch (-a1) a -> Just (newp a2, a2)
+           | amountsMatch (-a2) a -> Just (newp a1, a1)
+           | otherwise            -> Nothing
 
     -- Get the single-commodity costless amount from a conversion posting, or raise an error.
     conversionPostingAmountNoCost p = case postingSingleAmount p of
@@ -322,7 +413,8 @@ transactionInferCostsFromEquity dryrun acctTypes t = first (annotateErrorWithTra
         Just Amount{aprice=Just _} -> Left $ annotateWithPostings [p] "Conversion postings must not have a cost:"
         Nothing                    -> Left $ annotateWithPostings [p] "Conversion postings must have a single-commodity amount:"
 
-    amountMatches a b = acommodity a == acommodity b && aquantity a == aquantity b
+    -- Do these amounts look the same when compared at the first's display precision ?
+    amountsMatch a b = amountLooksZero $ amountSetPrecision (asprecision $ astyle a) $ a - b
 
     -- Delete a posting from the indexed list of postings based on either its
     -- index or its posting amount.
@@ -338,23 +430,33 @@ transactionInferCostsFromEquity dryrun acctTypes t = first (annotateErrorWithTra
     deleteUniqueMatch _ []                 = Nothing
     annotateWithPostings xs str = T.unlines $ str : postingsAsLines False xs
 
--- Using the provided account types map, sort the given indexed postings
+dbgShowAmountPrecision a =
+  case asprecision $ astyle a of
+    Precision n      -> show n
+    NaturalPrecision -> show $ decimalPlaces $ normalizeDecimal $ aquantity a
+
+-- Given the names of conversion equity accounts, sort the given indexed postings
 -- into three lists of posting numbers (stored in two pairs), like so:
--- (conversion postings, (costful postings, other postings)).
+-- (conversion postings, (costful other postings, costless other postings)).
 -- A true first argument activates its secondary function: check that all
 -- conversion postings occur in adjacent pairs, otherwise return an error.
-partitionAndCheckConversionPostings :: Bool -> M.Map AccountName AccountType -> [IdxPosting] -> Either Text ( [(IdxPosting, IdxPosting)], ([IdxPosting], [IdxPosting]) )
-partitionAndCheckConversionPostings check acctTypes = fmap fst . foldrM select (([], ([], [])), Nothing)
+partitionAndCheckConversionPostings :: Bool -> [AccountName] -> [IdxPosting] -> Either Text ( [(IdxPosting, IdxPosting)], ([IdxPosting], [IdxPosting]) )
+partitionAndCheckConversionPostings check conversionaccts =
+  -- Left fold processes postings in parse order, so that eg inferred costs
+  -- will be added to the first (top-most) posting, not the last one.
+  foldlM select (([], ([], [])), Nothing)
+    -- The costless other postings are somehow reversed still; "second (second reverse" fixes that.
+    <&> fmap (second (second reverse) . fst)
   where
-    select np@(_, p) ((cs, others@(ps, os)), Nothing)
+    select ((cs, others@(ps, os)), Nothing) np@(_, p)
       | isConversion p = Right ((cs, others),      Just np)
       | hasCost p      = Right ((cs, (np:ps, os)), Nothing)
       | otherwise      = Right ((cs, (ps, np:os)), Nothing)
-    select np@(_, p) ((cs, others@(ps,os)), Just lst)
+    select ((cs, others@(ps,os)), Just lst) np@(_, p)
       | isConversion p = Right (((lst, np):cs, others), Nothing)
       | check          = Left "Conversion postings must occur in adjacent pairs"
       | otherwise      = Right ((cs, (ps, np:os)), Nothing)
-    isConversion p = M.lookup (paccount p) acctTypes == Just Conversion
+    isConversion p = paccount p `elem` conversionaccts
     hasCost p = isJust $ aprice =<< postingSingleAmount p
 
 -- | Get a posting's amount if it is single-commodity.
@@ -379,6 +481,10 @@ transactionMapPostings f t@Transaction{tpostings=ps} = t{tpostings=map f ps}
 transactionMapPostingAmounts :: (MixedAmount -> MixedAmount) -> Transaction -> Transaction
 transactionMapPostingAmounts f  = transactionMapPostings (postingTransformAmount f)
 
+-- | All posting amounts from this transactin, in order.
+transactionAmounts :: Transaction -> [MixedAmount]
+transactionAmounts = map pamount . tpostings
+
 -- | The file path from which this transaction was parsed.
 transactionFile :: Transaction -> FilePath
 transactionFile Transaction{tsourcepos} = sourceName $ fst tsourcepos
