@@ -249,31 +249,48 @@ deriving instance Generic (DecimalRaw a)
 data AmountPrice = UnitPrice !Amount | TotalPrice !Amount
   deriving (Eq,Ord,Generic,Show)
 
--- | Display style for an amount.
+-- | Every Amount has one of these, influencing how the amount is displayed.
+-- Also, each Commodity can have one, which can be applied to its amounts for consistent display.
+-- See also Amount.AmountDisplayOpts.
 data AmountStyle = AmountStyle {
-      ascommodityside   :: !Side,                   -- ^ does the symbol appear on the left or the right ?
-      ascommodityspaced :: !Bool,                   -- ^ space between symbol and quantity ?
-      asprecision       :: !AmountPrecision,        -- ^ number of digits displayed after the decimal point
-      asdecimalpoint    :: !(Maybe Char),           -- ^ character used as decimal point: period or comma. Nothing means "unspecified, use default"
-      asdigitgroups     :: !(Maybe DigitGroupStyle) -- ^ style for displaying digit groups, if any
+  ascommodityside   :: !Side,                     -- ^ show the symbol on the left or the right ?
+  ascommodityspaced :: !Bool,                     -- ^ show a space between symbol and quantity ?
+  asdigitgroups     :: !(Maybe DigitGroupStyle),  -- ^ show the integer part with these digit group marks, or not
+  asdecimalmark     :: !(Maybe Char),             -- ^ show this character (should be . or ,) as decimal mark, or use the default (.)
+  asprecision       :: !AmountPrecision,          -- ^ "display precision" - show this number of digits after the decimal point
+  asrounding        :: !Rounding                  -- ^ "rounding strategy" - kept here for convenience, for now:
+                                                  --   when displaying an amount, it is ignored,
+                                                  --   but when applying this style to another amount, it determines 
+                                                  --   how hard we should try to adjust the amount's display precision.
 } deriving (Eq,Ord,Read,Generic)
 
 instance Show AmountStyle where
-  show AmountStyle{..} = concat
-    [ "AmountStylePP \""
+  show AmountStyle{..} = unwords
+    [ "AmountStylePP"
     , show ascommodityside
     , show ascommodityspaced
-    , show asprecision
-    , show asdecimalpoint
     , show asdigitgroups
-    , "..\""
+    , show asdecimalmark
+    , show asprecision
+    , show asrounding
     ]
 
 -- | The "display precision" for a hledger amount, by which we mean
 -- the number of decimal digits to display to the right of the decimal mark.
--- This can be from 0 to 255 digits (the maximum supported by the Decimal library),
--- or NaturalPrecision meaning "show all significant decimal digits".
-data AmountPrecision = Precision !Word8 | NaturalPrecision deriving (Eq,Ord,Read,Show,Generic)
+data AmountPrecision =
+    Precision !Word8    -- ^ show this many decimal digits (0..255)
+  | NaturalPrecision    -- ^ show all significant decimal digits stored internally
+  deriving (Eq,Ord,Read,Show,Generic)
+
+-- | "Rounding strategy" - how to apply an AmountStyle's display precision
+-- to a posting amount (and its cost, if any). 
+-- Mainly used to customise print's output, with --round=none|soft|hard|all.
+data Rounding =
+    NoRounding    -- ^ keep display precisions unchanged in amt and cost
+  | SoftRounding  -- ^ do soft rounding of amt and cost amounts (show more or fewer decimal zeros to approximate the target precision, but don't hide significant digits)
+  | HardRounding  -- ^ do hard rounding of amt (use the exact target precision, possibly hiding significant digits), and soft rounding of cost
+  | AllRounding   -- ^ do hard rounding of amt and cost
+  deriving (Eq,Ord,Read,Show,Generic)
 
 -- | A style for displaying digit groups in the integer part of a
 -- floating point number. It consists of the character used to
@@ -298,6 +315,24 @@ data Amount = Amount {
       aprice      :: !(Maybe AmountPrice)  -- ^ the (fixed, transaction-specific) price for this amount, if any
     } deriving (Eq,Ord,Generic,Show)
 
+-- | Types with this class have one or more amounts,
+-- which can have display styles applied to them.
+class HasAmounts a where
+  styleAmounts :: M.Map CommoditySymbol AmountStyle -> a -> a
+
+instance HasAmounts a =>
+  HasAmounts [a]
+  where styleAmounts styles = map (styleAmounts styles)
+
+instance (HasAmounts a, HasAmounts b) =>
+  HasAmounts (a,b)
+  where styleAmounts styles (aa,bb) = (styleAmounts styles aa, styleAmounts styles bb)
+
+instance HasAmounts a =>
+  HasAmounts (Maybe a)
+  where styleAmounts styles = fmap (styleAmounts styles)
+
+
 newtype MixedAmount = Mixed (M.Map MixedAmountKey Amount) deriving (Generic,Show)
 
 instance Eq  MixedAmount where a == b  = maCompare a b == EQ
