@@ -25,6 +25,7 @@ module Hledger.Data.Valuation (
   ,marketPriceReverse
   ,priceDirectiveToMarketPrice
   ,amountPriceDirectiveFromCost
+  ,valuationTypeValuationCommodity
   -- ,priceLookup
   ,tests_Valuation
 )
@@ -47,6 +48,7 @@ import Hledger.Data.Types
 import Hledger.Data.Amount
 import Hledger.Data.Dates (nulldate)
 import Text.Printf (printf)
+import Data.Decimal (decimalPlaces, roundTo)
 
 
 ------------------------------------------------------------------------------
@@ -67,6 +69,14 @@ data ValuationType =
   | AtDate Day (Maybe CommoditySymbol)  -- ^ convert to default or given valuation commodity, using market prices on some date
   deriving (Show,Eq)
 
+valuationTypeValuationCommodity :: ValuationType -> Maybe CommoditySymbol
+valuationTypeValuationCommodity = \case
+    AtThen   (Just c) -> Just c
+    AtEnd    (Just c) -> Just c
+    AtNow    (Just c) -> Just c
+    AtDate _ (Just c) -> Just c
+    _                 -> Nothing
+
 -- | A price oracle is a magic memoising function that efficiently
 -- looks up market prices (exchange rates) from one commodity to
 -- another (or if unspecified, to a default valuation commodity) on a
@@ -97,22 +107,19 @@ priceDirectiveToMarketPrice PriceDirective{..} =
              , mprate = aquantity pdamount
              }
 
--- | Make one or more `MarketPrice` from an 'Amount' and its price directives.
+-- | Infer a market price from the given amount and its cost (if any),
+-- and make a corresponding price directive on the given date.
+-- The price's display precision will be set to show all significant
+-- decimal digits; or if they seem to be infinite, defaultPrecisionLimit.
 amountPriceDirectiveFromCost :: Day -> Amount -> Maybe PriceDirective
-amountPriceDirectiveFromCost d amt@Amount{acommodity=fromcomm, aquantity=fromq} = case aprice amt of
-    Just (UnitPrice pa)               -> Just $ pd{pdamount=pa}
-    Just (TotalPrice pa) | fromq /= 0 -> Just $ pd{pdamount=fromq `divideAmountExtraPrecision` pa}
-    _                                 -> Nothing
+amountPriceDirectiveFromCost d amt@Amount{acommodity=fromcomm, aquantity=n} = case aprice amt of
+    Just (UnitPrice u)           -> Just $ pd{pdamount=u}
+    Just (TotalPrice t) | n /= 0 -> Just $ pd{pdamount=u}
+      where u = amountSetFullPrecisionOr Nothing $ divideAmount n t
+    _                            -> Nothing
   where
     pd = PriceDirective{pddate = d, pdcommodity = fromcomm, pdamount = nullamt}
 
-    divideAmountExtraPrecision n a = (n `divideAmount` a) { astyle = style' }
-      where
-        style' = (astyle a) { asprecision = precision' }
-        precision' = case asprecision (astyle a) of
-                          NaturalPrecision -> NaturalPrecision
-                          Precision p      -> Precision $ (numDigitsInt $ truncate n) + p
-
 ------------------------------------------------------------------------------
 -- Converting things to value
 
@@ -129,7 +136,7 @@ mixedAmountApplyValuation priceoracle styles periodlast today postingdate v =
 
 -- | Convert an Amount to its cost if requested, and style it appropriately.
 amountToCost :: M.Map CommoditySymbol AmountStyle -> ConversionOp -> Amount -> Amount
-amountToCost styles ToCost         = styleAmount styles . amountCost
+amountToCost styles ToCost         = styleAmounts styles . amountCost
 amountToCost _      NoConversionOp = id
 
 -- | Apply a specified valuation to this amount, using the provided
@@ -179,22 +186,32 @@ mixedAmountValueAtDate priceoracle styles mc d = mapMixedAmount (amountValueAtDa
 -- valuation date.)
 --
 -- The returned amount will have its commodity's canonical style applied,
--- but with the precision adjusted to show all significant decimal digits
--- up to a maximum of 8. (experimental)
+-- (with soft display rounding).
 --
 -- If the market prices available on that date are not sufficient to
 -- calculate this value, the amount is left unchanged.
+--
 amountValueAtDate :: PriceOracle -> M.Map CommoditySymbol AmountStyle -> Maybe CommoditySymbol -> Day -> Amount -> Amount
 amountValueAtDate priceoracle styles mto d a =
+  let lbl = lbl_ "amountValueAtDate" in
   case priceoracle (d, acommodity a, mto) of
     Nothing           -> a
     Just (comm, rate) ->
-      -- setNaturalPrecisionUpTo 8 $  -- XXX force higher precision in case amount appears to be zero ?
-                                      -- Make default display style use precision 2 instead of 0 ?
-                                      -- Leave as is for now; mentioned in manual.
-      styleAmount styles
       nullamt{acommodity=comm, aquantity=rate * aquantity a}
 
+      -- Manage style and precision of the new amount. Initially:
+      --  rate is a Decimal with the internal precision of the original market price declaration.
+      --  aquantity is a Decimal with a's internal precision.
+      --  The calculated value's internal precision may be different from these.
+      --  Its display precision will be that of nullamt (0).
+      -- Now apply the standard display style for comm (if there is one)
+      & styleAmounts styles
+      -- set the display precision to match the internal precision (showing all digits),
+      -- unnormalised (don't strip trailing zeros);
+      -- but if it looks like an infinite decimal, limit the precision to 8.
+      & amountSetFullPrecisionOr Nothing
+      & dbg9With (lbl "calculated value".showAmount)
+
 -- | Calculate the gain of each component amount, that is the difference
 -- between the valued amount and the value of the cost basis (see
 -- mixedAmountApplyValuation).
@@ -266,7 +283,19 @@ priceLookup makepricegraph d from mto =
         of
           Nothing -> Nothing
           Just [] -> Nothing
-          Just ps -> Just (mpto $ last ps, product $ map mprate ps)
+          Just ps -> Just (mpto $ last ps, rate)
+            where
+              rates = map mprate ps
+              rate =
+                -- aggregate all the prices into one
+                product rates
+                -- product (Decimal's Num instance) normalises, stripping trailing zeros.
+                -- Here we undo that (by restoring the old max precision with roundTo), 
+                -- so that amountValueAtDate can see the original internal precision,
+                -- to use as the display precision of calculated value amounts.
+                -- (This can add more than the original number of trailing zeros to some prices,
+                -- making them seem more precise than they were, but it seems harmless here.)
+                & roundTo (maximum $ map decimalPlaces rates)
 
 tests_priceLookup =
   let
@@ -335,6 +364,12 @@ pricesShortestPath start end edges =
       case concatMap extend paths of
         [] -> Nothing 
         _ | pathlength > maxpathlength -> 
+          -- XXX This is unusual:
+          -- 1. A warning, not an error, which we usually avoid
+          -- 2. Not a debug message (when triggered, we always print it)
+          -- 3. Printed either to stdout or (eg in hledger-ui) to the debug log file.
+          -- This is the only place we use traceOrLog like this.
+          -- Also before 1.32.2, traceOrLog was doing the opposite of what it should [#2134].
           traceOrLog ("gave up searching for a price chain at length "++show maxpathlength++", please report a bug")
           Nothing
           where 
@@ -435,6 +470,8 @@ makePriceGraph alldeclaredprices allinferredprices d =
     ,pgDefaultValuationCommodities=defaultdests
     }
   where
+    -- XXX logic duplicated in Hledger.Cli.Commands.Prices.prices, keep synced
+
     -- prices in effect on date d, either declared or inferred
     visibledeclaredprices = dbg9 "visibledeclaredprices" $ filter ((<=d).mpdate) alldeclaredprices
     visibleinferredprices = dbg9 "visibleinferredprices" $ filter ((<=d).mpdate) allinferredprices
