@@ -8,12 +8,80 @@ Journals from various data formats. Use this module if you want to parse
 journal data or read journal files. Generally it should not be necessary
 to import modules below this one.
 
+== Journal reading
+
+Reading an input file (in journal, csv, timedot, or timeclock format..)
+involves these steps:
+
+- select an appropriate file format "reader"
+  based on filename extension/file path prefix/function parameter.
+  A reader contains a parser and a finaliser (usually @journalFinalise@).
+
+- run the parser to get a ParsedJournal
+  (this may run additional sub-parsers to parse included files)
+
+- run the finaliser to get a complete Journal, which passes standard checks
+
+- if reading multiple files: merge the per-file Journals into one
+  overall Journal
+
+- if using -s/--strict: run additional strict checks
+
+- if running print --new: save .latest files for each input file.
+  (import also does this, as its final step.)
+
+== Journal merging
+
+Journal implements the Semigroup class, so two Journals can be merged
+into one Journal with @j1 <> j2@. This is implemented by the
+@journalConcat@ function, whose documentation explains what merging
+Journals means exactly.
+
+== Journal finalising
+
+This is post-processing done after parsing an input file, such as
+inferring missing information, normalising amount styles,
+checking for errors and so on - a delicate and influential stage
+of data processing. 
+In hledger it is done by @journalFinalise@, which converts a
+preliminary ParsedJournal to a validated, ready-to-use Journal.
+This is called immediately after the parsing of each input file.
+It is not called when Journals are merged.
+
+== Journal reading API
+
+There are three main Journal-reading functions:
+
+- readJournal to read from a Text value.
+  Selects a reader and calls its parser and finaliser,
+  then does strict checking if needed.
+
+- readJournalFile to read one file, or stdin if the file path is @-@.
+  Uses the file path/file name to help select the reader,
+  calls readJournal,
+  then writes .latest files if needed.
+
+- readJournalFiles to read multiple files.
+  Calls readJournalFile for each file (without strict checking or .latest file writing)
+  then merges the Journals into one,
+  then does strict checking and .latest file writing at the end if needed.
+
+Each of these also has an easier variant with ' suffix,
+which uses default options and has a simpler type signature.
+
+One more variant, @readJournalFilesAndLatestDates@, is like
+readJournalFiles but exposing the latest transaction date
+(and how many on the same day) seen for each file.
+This is used by the import command.
+
 -}
 
 --- ** language
 {-# LANGUAGE OverloadedStrings   #-}
 {-# LANGUAGE PackageImports      #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE NamedFieldPuns #-}
+{-# OPTIONS_GHC -Wno-unrecognised-pragmas #-}
 
 --- ** exports
 module Hledger.Read (
@@ -26,10 +94,11 @@ module Hledger.Read (
   ensureJournalFileExists,
 
   -- * Journal parsing
+  runExceptT,
   readJournal,
   readJournalFile,
   readJournalFiles,
-  runExceptT,
+  readJournalFilesAndLatestDates,
 
   -- * Easy journal parsing
   readJournal',
@@ -37,6 +106,11 @@ module Hledger.Read (
   readJournalFiles',
   orDieTrying,
 
+  -- * Misc
+  journalStrictChecks,
+  saveLatestDates,
+  saveLatestDatesForFiles,
+
   -- * Re-exported
   JournalReader.tmpostingrulep,
   findReader,
@@ -53,13 +127,13 @@ module Hledger.Read (
 --- ** imports
 import qualified Control.Exception as C
 import Control.Monad (unless, when)
-import "mtl" Control.Monad.Except (ExceptT(..), runExceptT)
+import "mtl" Control.Monad.Except (ExceptT(..), runExceptT, liftEither)
 import Control.Monad.IO.Class (MonadIO, liftIO)
 import Data.Default (def)
 import Data.Foldable (asum)
 import Data.List (group, sort, sortBy)
 import Data.List.NonEmpty (nonEmpty)
-import Data.Maybe (fromMaybe)
+import Data.Maybe (catMaybes, fromMaybe)
 import Data.Ord (comparing)
 import Data.Semigroup (sconcat)
 import Data.Text (Text)
@@ -85,6 +159,7 @@ import Hledger.Read.RulesReader (tests_RulesReader)
 -- import Hledger.Read.TimeclockReader (tests_TimeclockReader)
 import Hledger.Utils
 import Prelude hiding (getContents, writeFile)
+import Hledger.Data.JournalChecks (journalCheckAccounts, journalCheckCommodities)
 
 --- ** doctest setup
 -- $setup
@@ -126,7 +201,8 @@ type PrefixedFilePath = FilePath
 
 -- | @readJournal iopts mfile txt@
 --
--- Read a Journal from some text, or return an error message.
+-- Read a Journal from some text, with strict checks if enabled,
+-- or return an error message.
 --
 -- The reader (data format) is chosen based on, in this order:
 --
@@ -137,16 +213,19 @@ type PrefixedFilePath = FilePath
 -- - a file extension in @mfile@
 --
 -- If none of these is available, or if the reader name is unrecognised,
--- we use the journal reader. (We used to try all readers in this case;
--- since hledger 1.17, we prefer predictability.)
+-- we use the journal reader (for predictability).
+--
 readJournal :: InputOpts -> Maybe FilePath -> Text -> ExceptT String IO Journal
-readJournal iopts mpath txt = do
+readJournal iopts@InputOpts{strict_} mpath txt = do
   let r :: Reader IO = fromMaybe JournalReader.reader $ findReader (mformat_ iopts) mpath
   dbg6IO "readJournal: trying reader" (rFormat r)
-  rReadFn r iopts (fromMaybe "(string)" mpath) txt
+  j <- rReadFn r iopts (fromMaybe "(string)" mpath) txt
+  when strict_ $ liftEither $ journalStrictChecks j
+  return j
 
 -- | Read a Journal from this file, or from stdin if the file path is -,
--- or return an error message. The file path can have a READER: prefix.
+-- with strict checks if enabled, or return an error message.
+-- The file path can have a READER: prefix.
 --
 -- The reader (data format) to use is determined from (in priority order):
 -- the @mformat_@ specified in the input options, if any;
@@ -156,8 +235,27 @@ readJournal iopts mpath txt = do
 --
 -- The input options can also configure balance assertion checking, automated posting
 -- generation, a rules file for converting CSV data, etc.
+--
+-- If using --new, and if latest-file writing is enabled in input options,
+-- and after passing strict checks if enabled, a .latest.FILE file will be created/updated
+-- (for the main file only, not for included files),
+-- to remember the latest transaction date (and how many transactions on this date)
+-- successfully read.
+--
 readJournalFile :: InputOpts -> PrefixedFilePath -> ExceptT String IO Journal
-readJournalFile iopts prefixedfile = do
+readJournalFile iopts@InputOpts{new_, new_save_} prefixedfile = do
+  (j, mlatestdates) <- readJournalFileAndLatestDates iopts prefixedfile
+  when (new_ && new_save_) $ liftIO $
+    case mlatestdates of
+      Nothing                        -> return ()
+      Just (LatestDatesForFile f ds) -> saveLatestDates ds f
+  return j
+
+-- The implementation of readJournalFile.
+-- With --new, it also returns the latest transaction date(s) read from each file.
+-- readJournalFiles uses this to update .latest files only after a successful read of all.
+readJournalFileAndLatestDates :: InputOpts -> PrefixedFilePath -> ExceptT String IO (Journal, Maybe LatestDatesForFile)
+readJournalFileAndLatestDates iopts prefixedfile = do
   let
     (mfmt, f) = splitReaderPrefix prefixedfile
     iopts' = iopts{mformat_=asum [mfmt, mformat_ iopts]}
@@ -168,24 +266,49 @@ readJournalFile iopts prefixedfile = do
     -- <- T.readFile f  -- or without line ending translation, for testing
   j <- readJournal iopts' (Just f) t
   if new_ iopts
-     then do
-       ds <- liftIO $ previousLatestDates f
-       let (newj, newds) = journalFilterSinceLatestDates ds j
-       when (new_save_ iopts && not (null newds)) . liftIO $ saveLatestDates newds f
-       return newj
-     else return j
+    then do
+      ds <- liftIO $ previousLatestDates f
+      let (newj, newds) = journalFilterSinceLatestDates ds j
+      return (newj, Just $ LatestDatesForFile f newds)
+    else
+      return (j, Nothing)
 
--- | Read a Journal from each specified file path and combine them into one.
--- Or, return the first error message.
+-- | Read a Journal from each specified file path (using @readJournalFile@) 
+-- and combine them into one; or return the first error message.
+-- Strict checks, if enabled, are deferred till the end.
+-- Writing .latest files, if enabled, is also deferred till the end,
+-- and happens only if strict checks pass.
 --
 -- Combining Journals means concatenating them, basically.
 -- The parse state resets at the start of each file, which means that
 -- directives & aliases do not affect subsequent sibling or parent files.
 -- They do affect included child files though.
 -- Also the final parse state saved in the Journal does span all files.
+--
 readJournalFiles :: InputOpts -> [PrefixedFilePath] -> ExceptT String IO Journal
-readJournalFiles iopts =
-  fmap (maybe def sconcat . nonEmpty) . mapM (readJournalFile iopts)
+readJournalFiles iopts@InputOpts{strict_,new_,new_save_} prefixedfiles = do
+  let iopts' = iopts{strict_=False, new_save_=False}
+  (j, latestdatesforfiles) <-
+    traceOrLogAt 6 ("readJournalFiles: "++show prefixedfiles) $
+    readJournalFilesAndLatestDates iopts' prefixedfiles
+  when strict_ $ liftEither $ journalStrictChecks j
+  when (new_ && new_save_) $ liftIO $ saveLatestDatesForFiles latestdatesforfiles
+  return j
+
+-- The implementation of readJournalFiles, but with --new, 
+-- also returns the latest transaction date(s) read in each file.
+-- Used by the import command, to save those at the end.
+readJournalFilesAndLatestDates :: InputOpts -> [PrefixedFilePath] -> ExceptT String IO (Journal, [LatestDatesForFile])
+readJournalFilesAndLatestDates iopts pfs = do
+  (js, lastdates) <- unzip <$> mapM (readJournalFileAndLatestDates iopts) pfs
+  return (maybe def sconcat $ nonEmpty js, catMaybes lastdates)
+
+-- | Run the extra -s/--strict checks on a journal,
+-- returning the first error message if any of them fail.
+journalStrictChecks :: Journal -> Either String ()
+journalStrictChecks j = do
+  journalCheckAccounts j
+  journalCheckCommodities j
 
 -- | An easy version of 'readJournal' which assumes default options, and fails
 -- in the IO monad.
@@ -251,17 +374,25 @@ newJournalContent = do
 -- and how many transactions there were on that date.
 type LatestDates = [Day]
 
+-- The path of an input file, and its current "LatestDates".
+data LatestDatesForFile = LatestDatesForFile FilePath LatestDates
+
 -- | Get all instances of the latest date in an unsorted list of dates.
 -- Ie, if the latest date appears once, return it in a one-element list,
 -- if it appears three times (anywhere), return three of it.
 latestDates :: [Day] -> LatestDates
-latestDates = headDef [] . take 1 . group . reverse . sort
+latestDates = {-# HLINT ignore "Avoid reverse" #-}
+  headDef [] . take 1 . group . reverse . sort
 
--- | Remember that these transaction dates were the latest seen when
--- reading this journal file.
+-- | Save the given latest date(s) seen in the given data FILE,
+-- in a hidden file named .latest.FILE, creating it if needed.
 saveLatestDates :: LatestDates -> FilePath -> IO ()
 saveLatestDates dates f = T.writeFile (latestDatesFileFor f) $ T.unlines $ map showDate dates
 
+-- | Save each file's latest dates.
+saveLatestDatesForFiles :: [LatestDatesForFile] -> IO ()
+saveLatestDatesForFiles = mapM_ (\(LatestDatesForFile f ds) -> saveLatestDates ds f)
+
 -- | What were the latest transaction dates seen the last time this
 -- journal file was read ? If there were multiple transactions on the
 -- latest date, that number of dates is returned, otherwise just one.
@@ -284,9 +415,6 @@ latestDatesFileFor f = dir </> ".latest" <.> fname
   where
     (dir, fname) = splitFileName f
 
-readFileStrictly :: FilePath -> IO Text
-readFileStrictly f = readFilePortably f >>= \t -> C.evaluate (T.length t) >> return t
-
 -- | Given zero or more latest dates (all the same, representing the
 -- latest previously seen transaction date, and how many transactions
 -- were seen on that date), remove transactions with earlier dates
