@@ -490,7 +490,7 @@ commoditydirectiveonelinep = do
   lift skipNonNewlineSpaces
   _ <- lift followingcommentp
   let comm = Commodity{csymbol=acommodity, cformat=Just $ dbg6 "style from commodity directive" astyle}
-  if isNothing $ asdecimalpoint astyle
+  if isNothing $ asdecimalmark astyle
   then customFailure $ parseErrorAt off pleaseincludedecimalpoint
   else modify' (\j -> j{jcommodities=M.insert acommodity comm $ jcommodities j})
 
@@ -533,7 +533,7 @@ formatdirectivep expectedsym = do
   _ <- lift followingcommentp
   if acommodity==expectedsym
     then
-      if isNothing $ asdecimalpoint astyle
+      if isNothing $ asdecimalmark astyle
       then customFailure $ parseErrorAt off pleaseincludedecimalpoint
       else return $ dbg6 "style from format subdirective" astyle
     else customFailure $ parseErrorAt off $
@@ -629,7 +629,7 @@ payeedirectivep :: JournalParser m ()
 payeedirectivep = do
   string "payee" <?> "payee directive"
   lift skipNonNewlineSpaces1
-  payee <- lift $ T.strip <$> noncommenttext1p
+  payee <- lift $ T.strip <$> (try doublequotedtextp <|> noncommenttext1p)
   (comment, tags) <- lift transactioncommentp
   skipMany indentedlinep
   addPayeeDeclaration (payee, comment, tags)
@@ -648,7 +648,7 @@ defaultcommoditydirectivep = do
   off <- getOffset
   Amount{acommodity,astyle} <- amountp
   lift restofline
-  if isNothing $ asdecimalpoint astyle
+  if isNothing $ asdecimalmark astyle
   then customFailure $ parseErrorAt off pleaseincludedecimalpoint
   else setDefaultCommodityAndStyle (acommodity, astyle)
 
@@ -1134,8 +1134,10 @@ tests_JournalReader = testGroup "JournalReader" [
       }
 
   ,testGroup "payeedirectivep" [
-       testCase "simple"             $ assertParse payeedirectivep "payee foo\n"
+        testCase "simple"             $ assertParse payeedirectivep "payee foo\n"
        ,testCase "with-comment"       $ assertParse payeedirectivep "payee foo ; comment\n"
+       ,testCase "double-quoted"      $ assertParse payeedirectivep "payee \"a b\"\n"
+       ,testCase "empty        "      $ assertParse payeedirectivep "payee \"\"\n"
        ]
 
   ,testCase "tagdirectivep" $ do
