@@ -69,15 +69,16 @@ import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
 import Data.Foldable (asum, toList)
 import Text.Megaparsec hiding (match, parse)
-import Text.Megaparsec.Char (char, newline, string)
+import Text.Megaparsec.Char (char, newline, string, digitChar)
 import Text.Megaparsec.Custom (parseErrorAt)
 import Text.Printf (printf)
 
 import Hledger.Data
 import Hledger.Utils
-import Hledger.Read.Common (aliasesFromOpts, Reader(..), InputOpts(..), amountp, statusp, journalFinalise, accountnamep )
+import Hledger.Read.Common (aliasesFromOpts, Reader(..), InputOpts(..), amountp, statusp, journalFinalise, accountnamep, commenttagsp )
 import Hledger.Read.CsvUtils
 import System.Directory (doesFileExist, getHomeDirectory)
+import Data.Either (fromRight)
 
 --- ** doctest setup
 -- $setup
@@ -258,7 +259,7 @@ data CsvRules' a = CsvRules' {
 type CsvRulesParsed = CsvRules' ()
 
 -- | Type used after parsing is done. Directives, assignments and conditional blocks
--- are in the same order as they were in the unput file and rblocksassigning is functional.
+-- are in the same order as they were in the input file and rblocksassigning is functional.
 -- Ready to be used for CSV record processing
 type CsvRules = CsvRules' (Text -> [ConditionalBlock])  -- XXX simplify
 
@@ -296,11 +297,14 @@ type HledgerFieldName = Text
 -- containing csv field references to be interpolated.
 type FieldTemplate    = Text
 
+-- | A reference to a regular expression match group. Eg \1.
+type MatchGroupReference = Text
+
 -- | A strptime date parsing pattern, as supported by Data.Time.Format.
 type DateFormat       = Text
 
 -- | A prefix for a matcher test, either & or none (implicit or).
-data MatcherPrefix = And | None
+data MatcherPrefix = And | Not | None
   deriving (Show, Eq)
 
 -- | A single test for matching a CSV record, in one way or another.
@@ -372,10 +376,12 @@ JOURNAL-PSEUDO-FIELD: amount-in | amount-out | currency
 
 ASSIGNMENT-SEPARATOR: SPACE | ( : SPACE? )
 
-FIELD-VALUE: VALUE (possibly containing CSV-FIELD-REFERENCEs)
+FIELD-VALUE: VALUE (possibly containing CSV-FIELD-REFERENCEs and REGEX-MATCHGROUP-REFERENCEs)
 
 CSV-FIELD-REFERENCE: % CSV-FIELD
 
+REGEX-MATCHGROUP-REFERENCE: \ DIGIT+
+
 CSV-FIELD: ( FIELD-NAME | FIELD-NUMBER ) (corresponding to a CSV field)
 
 FIELD-NUMBER: DIGIT+
@@ -660,7 +666,7 @@ fieldmatcherp end = do
 matcherprefixp :: CsvRulesParser MatcherPrefix
 matcherprefixp = do
   lift $ dbgparse 8 "trying matcherprefixp"
-  (char '&' >> lift skipNonNewlineSpaces >> return And) <|> return None
+  (char '&' >> lift skipNonNewlineSpaces >> return And) <|> (char '!' >> lift skipNonNewlineSpaces >> return Not) <|> return None
 
 csvfieldreferencep :: CsvRulesParser CsvFieldReference
 csvfieldreferencep = do
@@ -707,7 +713,11 @@ hledgerField = getEffectiveAssignment
 -- | Look up the final value assigned to a hledger field, with csv field
 -- references interpolated.
 hledgerFieldValue :: CsvRules -> CsvRecord -> HledgerFieldName -> Maybe Text
-hledgerFieldValue rules record = fmap (renderTemplate rules record) . hledgerField rules record
+hledgerFieldValue rules record f = (fmap (renderTemplate rules record f) . hledgerField rules record) f
+
+maybeNegate :: MatcherPrefix -> Bool -> Bool
+maybeNegate Not origbool = not origbool
+maybeNegate _ origbool = origbool
 
 -- | Given the conversion rules, a CSV record and a hledger field name, find
 -- the value template ultimately assigned to this field, if any, by a field
@@ -721,60 +731,97 @@ getEffectiveAssignment rules record f = lastMay $ map snd $ assignments
   where
     -- all active assignments to field f, in order
     assignments = dbg9 "csv assignments" $ filter ((==f).fst) $ toplevelassignments ++ conditionalassignments
-      where
-        -- all top level field assignments
-        toplevelassignments    = rassignments rules
-        -- all field assignments in conditional blocks assigning to field f and active for the current csv record
-        conditionalassignments = concatMap cbAssignments $ filter isBlockActive $ (rblocksassigning rules) f
-          where
-            -- does this conditional block match the current csv record ?
-            isBlockActive :: ConditionalBlock -> Bool
-            isBlockActive CB{..} = any (all matcherMatches) $ groupedMatchers cbMatchers
-              where
-                -- does this individual matcher match the current csv record ?
-                matcherMatches :: Matcher -> Bool
-                matcherMatches (RecordMatcher _ pat) = regexMatchText pat' wholecsvline
-                  where
-                    pat' = dbg7 "regex" pat
-                    -- A synthetic whole CSV record to match against. Note, this can be
-                    -- different from the original CSV data:
-                    -- - any whitespace surrounding field values is preserved
-                    -- - any quotes enclosing field values are removed
-                    -- - and the field separator is always comma
-                    -- which means that a field containing a comma will look like two fields.
-                    wholecsvline = dbg7 "wholecsvline" $ T.intercalate "," record
-                matcherMatches (FieldMatcher _ csvfieldref pat) = regexMatchText pat csvfieldvalue
-                  where
-                    -- the value of the referenced CSV field to match against.
-                    csvfieldvalue = dbg7 "csvfieldvalue" $ replaceCsvFieldReference rules record csvfieldref
+    -- all top level field assignments
+    toplevelassignments    = rassignments rules 
+    -- all field assignments in conditional blocks assigning to field f and active for the current csv record
+    conditionalassignments = concatMap cbAssignments $ filter (isBlockActive rules record) $ (rblocksassigning rules) f
 
-                -- | Group matchers into associative pairs based on prefix, e.g.:
-                --   A
-                --   & B
-                --   C
-                --   D
-                --   & E
-                --   => [[A, B], [C], [D, E]]
-                groupedMatchers :: [Matcher] -> [[Matcher]]
-                groupedMatchers [] = []
-                groupedMatchers (x:xs) = (x:ys) : groupedMatchers zs
-                  where
-                    (ys, zs) = span (\y -> matcherPrefix y == And) xs
-                    matcherPrefix :: Matcher -> MatcherPrefix
-                    matcherPrefix (RecordMatcher prefix _) = prefix
-                    matcherPrefix (FieldMatcher prefix _ _) = prefix
+-- does this conditional block match the current csv record ?
+isBlockActive :: CsvRules -> CsvRecord -> ConditionalBlock -> Bool
+isBlockActive rules record CB{..} = any (all matcherMatches) $ groupedMatchers cbMatchers
+  where
+    -- does this individual matcher match the current csv record ?
+    matcherMatches :: Matcher -> Bool
+    matcherMatches (RecordMatcher prefix pat) = maybeNegate prefix origbool
+      where
+        pat' = dbg7 "regex" pat
+        -- A synthetic whole CSV record to match against. Note, this can be
+        -- different from the original CSV data:
+        -- - any whitespace surrounding field values is preserved
+        -- - any quotes enclosing field values are removed
+        -- - and the field separator is always comma
+        -- which means that a field containing a comma will look like two fields.
+        wholecsvline = dbg7 "wholecsvline" $ T.intercalate "," record
+        origbool = regexMatchText pat' wholecsvline
+    matcherMatches (FieldMatcher prefix csvfieldref pat) = maybeNegate prefix origbool
+      where
+        -- the value of the referenced CSV field to match against.
+        csvfieldvalue = dbg7 "csvfieldvalue" $ replaceCsvFieldReference rules record csvfieldref
+        origbool = regexMatchText pat csvfieldvalue
+
+    -- | Group matchers into associative pairs based on prefix, e.g.:
+    --   A
+    --   & B
+    --   C
+    --   D
+    --   & E
+    --   => [[A, B], [C], [D, E]]
+    groupedMatchers :: [Matcher] -> [[Matcher]]
+    groupedMatchers [] = []
+    groupedMatchers (x:xs) = (x:ys) : groupedMatchers zs
+      where
+        (ys, zs) = span (\y -> matcherPrefix y == And) xs
+        matcherPrefix :: Matcher -> MatcherPrefix
+        matcherPrefix (RecordMatcher prefix _) = prefix
+        matcherPrefix (FieldMatcher prefix _ _) = prefix
 
 -- | Render a field assignment's template, possibly interpolating referenced
--- CSV field values. Outer whitespace is removed from interpolated values.
-renderTemplate ::  CsvRules -> CsvRecord -> FieldTemplate -> Text
-renderTemplate rules record t = maybe t mconcat $ parseMaybe
-    (many $ takeWhile1P Nothing (/='%')
-        <|> replaceCsvFieldReference rules record <$> referencep)
+-- CSV field values or match groups. Outer whitespace is removed from interpolated values.
+renderTemplate ::  CsvRules -> CsvRecord -> HledgerFieldName -> FieldTemplate -> Text
+renderTemplate rules record f t =
+  maybe t mconcat $ parseMaybe
+    (many
+      (   literaltextp
+      <|> (matchrefp <&> replaceRegexGroupReference rules record f)
+      <|> (fieldrefp <&> replaceCsvFieldReference   rules record)
+      )
+    )
     t
   where
-    referencep = liftA2 T.cons (char '%') (takeWhile1P (Just "reference") isFieldNameChar) :: Parsec HledgerParseErrorData Text Text
+    literaltextp :: SimpleTextParser Text
+    literaltextp = some (nonBackslashOrPercent <|> nonRefBackslash <|> nonRefPercent) <&> T.pack
+      where
+        nonBackslashOrPercent = noneOf ['\\', '%'] <?> "character other than backslash or percent"
+        nonRefBackslash = try (char '\\' <* notFollowedBy digitChar) <?> "backslash that does not begin a match group reference"
+        nonRefPercent   = try (char '%'  <* notFollowedBy (satisfy isFieldNameChar)) <?> "percent that does not begin a field reference"
+    matchrefp    = liftA2 T.cons (char '\\') (takeWhile1P (Just "matchref")  isDigit)
+    fieldrefp    = liftA2 T.cons (char '%')  (takeWhile1P (Just "reference") isFieldNameChar)
     isFieldNameChar c = isAlphaNum c || c == '_' || c == '-'
 
+-- | Replace something that looks like a Regex match group reference with the
+-- resulting match group value after applying the Regex.
+replaceRegexGroupReference :: CsvRules -> CsvRecord -> HledgerFieldName -> MatchGroupReference -> Text
+replaceRegexGroupReference rules record f s = case T.uncons s of
+    Just ('\\', group) -> fromMaybe "" $ regexMatchValue rules record f group
+    _                  -> s
+
+regexMatchValue :: CsvRules -> CsvRecord -> HledgerFieldName -> Text -> Maybe Text
+regexMatchValue rules record f sgroup = let
+  matchgroups  = concatMap (getMatchGroups rules record)
+               $ concatMap cbMatchers
+               $ filter (isBlockActive rules record)
+               $ rblocksassigning rules f
+  group = (read (T.unpack sgroup) :: Int) - 1 -- adjust to 0-indexing
+  in atMay matchgroups group
+
+getMatchGroups :: CsvRules -> CsvRecord -> Matcher -> [Text]
+getMatchGroups _ record (RecordMatcher _ regex)  = let
+  txt = T.intercalate "," record -- see caveats of wholecsvline, in `isBlockActive`
+  in regexMatchTextGroups regex txt
+getMatchGroups rules record (FieldMatcher _ fieldref regex) = let
+  txt = replaceCsvFieldReference rules record fieldref
+  in regexMatchTextGroups regex txt
+
 -- | Replace something that looks like a reference to a csv field ("%date" or "%1)
 -- with that field's value. If it doesn't look like a field reference, or if we
 -- can't find such a field, replace it with the empty string.
@@ -1032,6 +1079,7 @@ transactionFromCsvRecord timesarezoned mtzin tzout sourcepos rules record = t
     code        = maybe "" singleline' $ fieldval "code"
     description = maybe "" singleline' $ fieldval "description"
     comment     = maybe "" unescapeNewlines $ fieldval "comment"
+    ttags       = fromRight [] $ rtp commenttagsp comment
     precomment  = maybe "" unescapeNewlines $ fieldval "precomment"
 
     singleline' = T.unwords . filter (not . T.null) . map T.strip . T.lines
@@ -1044,6 +1092,7 @@ transactionFromCsvRecord timesarezoned mtzin tzout sourcepos rules record = t
     p1IsVirtual = (accountNamePostingType <$> fieldval "account1") == Just VirtualPosting
     ps = [p | n <- [1..maxpostings]
          ,let cmt  = maybe "" unescapeNewlines $ fieldval ("comment"<> T.pack (show n))
+         ,let ptags = fromRight [] $ rtp commenttagsp cmt
          ,let currency = fromMaybe "" (fieldval ("currency"<> T.pack (show n)) <|> fieldval "currency")
          ,let mamount  = getAmount rules record currency p1IsVirtual n
          ,let mbalance = getBalance rules record currency n
@@ -1056,6 +1105,7 @@ transactionFromCsvRecord timesarezoned mtzin tzout sourcepos rules record = t
                              ,ptransaction      = Just t
                              ,pbalanceassertion = mkBalanceAssertion rules record <$> mbalance
                              ,pcomment          = cmt
+                             ,ptags             = ptags
                              ,ptype             = accountNamePostingType acct
                              }
          ]
@@ -1071,6 +1121,7 @@ transactionFromCsvRecord timesarezoned mtzin tzout sourcepos rules record = t
           ,tcode             = code
           ,tdescription      = description
           ,tcomment          = comment
+          ,ttags             = ttags
           ,tprecedingcomment = precomment
           ,tpostings         = ps
           }
@@ -1153,7 +1204,7 @@ getAmount rules record currency p1IsVirtual n =
 
     -- assignments to any of these field names with non-empty values
     assignments = [(f,a') | f <- fieldnames
-                          , Just v <- [T.strip . renderTemplate rules record <$> hledgerField rules record f]
+                          , Just v <- [T.strip . renderTemplate rules record f <$> hledgerField rules record f]
                           , not $ T.null v
                           -- XXX maybe ignore rule-generated values like "", "-", "$", "-$", "$-" ? cf CSV FORMAT -> "amount", "Setting amounts",
                           , let a = parseAmount rules record currency v
@@ -2977,6 +3028,12 @@ tests_RulesReader = testGroup "RulesReader" [
    ,let rules = mkrules $ defrules{rcsvfieldindexes=[("csvdate",1)], rconditionalblocks=[CB [FieldMatcher None "%csvdate" $ toRegex' "a"] [("date","%csvdate")]]}
     in testCase "conditional" $ getEffectiveAssignment rules ["a","b"] "date" @?= (Just "%csvdate")
 
+   ,let rules = mkrules $ defrules{rcsvfieldindexes=[("csvdate",1)], rconditionalblocks=[CB [FieldMatcher Not "%csvdate" $ toRegex' "a"] [("date","%csvdate")]]}
+    in testCase "negated-conditional-false" $ getEffectiveAssignment rules ["a","b"] "date" @?= (Nothing)
+  
+   ,let rules = mkrules $ defrules{rcsvfieldindexes=[("csvdate",1)], rconditionalblocks=[CB [FieldMatcher Not "%csvdate" $ toRegex' "b"] [("date","%csvdate")]]}
+    in testCase "negated-conditional-true" $ getEffectiveAssignment rules ["a","b"] "date" @?= (Just "%csvdate")
+
    ,let rules = mkrules $ defrules{rcsvfieldindexes=[("csvdate",1),("description",2)], rconditionalblocks=[CB [FieldMatcher None "%csvdate" $ toRegex' "a", FieldMatcher None "%description" $ toRegex' "b"] [("date","%csvdate")]]}
     in testCase "conditional-with-or-a" $ getEffectiveAssignment rules ["a"] "date" @?= (Just "%csvdate")
 
