@@ -1,4 +1,6 @@
-{-# LANGUAGE OverloadedStrings, RecordWildCards, FlexibleInstances #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE OverloadedStrings #-}
+
 {-|
 
 Generate several common kinds of report from a journal, as \"*Report\" -
@@ -13,7 +15,6 @@ module Hledger.Reports (
   module Hledger.Reports.ReportTypes,
   module Hledger.Reports.EntriesReport,
   module Hledger.Reports.PostingsReport,
-  module Hledger.Reports.TransactionsReport,
   module Hledger.Reports.AccountTransactionsReport,
   module Hledger.Reports.BalanceReport,
   module Hledger.Reports.MultiBalanceReport,
@@ -23,23 +24,21 @@ module Hledger.Reports (
 )
 where
 
+import Test.Tasty (testGroup)
 import Hledger.Reports.ReportOptions
 import Hledger.Reports.ReportTypes
 import Hledger.Reports.AccountTransactionsReport
 import Hledger.Reports.EntriesReport
 import Hledger.Reports.PostingsReport
-import Hledger.Reports.TransactionsReport
 import Hledger.Reports.BalanceReport
 import Hledger.Reports.MultiBalanceReport
 import Hledger.Reports.BudgetReport
-import Hledger.Utils.Test
 
-tests_Reports = tests "Reports" [
+tests_Reports = testGroup "Reports" [
    tests_BalanceReport
   ,tests_BudgetReport
   ,tests_AccountTransactionsReport
   ,tests_EntriesReport
   ,tests_MultiBalanceReport
   ,tests_PostingsReport
-  ,tests_ReportOptions
   ]
