@@ -83,6 +83,7 @@ import Text.Megaparsec.Custom
 import Hledger.Data
 import Hledger.Query
 import Hledger.Utils
+import Data.Function ((&))
 
 
 -- | What to calculate for each cell in a balance report.
@@ -156,6 +157,7 @@ data ReportOpts = ReportOpts {
     ,declared_         :: Bool  -- ^ Include accounts declared but not yet posted to ?
     ,row_total_        :: Bool
     ,no_total_         :: Bool
+    ,summary_only_     :: Bool
     ,show_costs_       :: Bool  -- ^ Show costs for reports which normally don't show them ?
     ,sort_amount_      :: Bool
     ,percent_          :: Bool
@@ -206,6 +208,7 @@ defreportopts = ReportOpts
     , declared_         = False
     , row_total_        = False
     , no_total_         = False
+    , summary_only_     = False
     , show_costs_       = False
     , sort_amount_      = False
     , percent_          = False
@@ -259,6 +262,7 @@ rawOptsToReportOpts d rawopts =
           ,declared_         = boolopt "declared" rawopts
           ,row_total_        = boolopt "row-total" rawopts
           ,no_total_         = boolopt "no-total" rawopts
+          ,summary_only_     = boolopt "summary-only" rawopts
           ,show_costs_       = boolopt "show-costs" rawopts
           ,sort_amount_      = boolopt "sort-amount" rawopts
           ,percent_          = boolopt "percent" rawopts
@@ -584,19 +588,28 @@ journalValueAndFilterPostingsWith rspec@ReportSpec{_rsQuery=q, _rsReportOpts=rop
 -- condition.
 journalApplyValuationFromOpts :: ReportSpec -> Journal -> Journal
 journalApplyValuationFromOpts rspec j =
-    journalApplyValuationFromOptsWith rspec j priceoracle
+  journalApplyValuationFromOptsWith rspec j priceoracle
   where priceoracle = journalPriceOracle (infer_prices_ $ _rsReportOpts rspec) j
 
 -- | Like journalApplyValuationFromOpts, but takes PriceOracle as an argument.
 journalApplyValuationFromOptsWith :: ReportSpec -> Journal -> PriceOracle -> Journal
 journalApplyValuationFromOptsWith rspec@ReportSpec{_rsReportOpts=ropts} j priceoracle =
-    case balancecalc_ ropts of
-      CalcGain -> journalMapPostings (\p -> postingTransformAmount (gain p) p) j
-      _        -> journalMapPostings (\p -> postingTransformAmount (valuation p) p) $ costing j
+  costfn j
+  & journalMapPostings (\p -> p
+    & dbg9With (lbl "before calc".showMixedAmountOneLine.pamount)
+    & postingTransformAmount (calcfn p)
+    & dbg9With (lbl (show calc).showMixedAmountOneLine.pamount)
+    )
   where
-    valuation p = maybe id (mixedAmountApplyValuation priceoracle styles (postingperiodend p) (_rsDay rspec) (postingDate p)) (value_ ropts)
-    gain      p = maybe id (mixedAmountApplyGain      priceoracle styles (postingperiodend p) (_rsDay rspec) (postingDate p)) (value_ ropts)
-    costing     = journalToCost (fromMaybe NoConversionOp $ conversionop_ ropts)
+    lbl = lbl_ "journalApplyValuationFromOptsWith"
+    -- Which custom calculation to do for balance reports. For all other reports, it will be CalcChange.
+    calc = balancecalc_ ropts
+    calcfn = case calc of
+      CalcGain -> \p -> maybe id (mixedAmountApplyGain      priceoracle styles (postingperiodend p) (_rsDay rspec) (postingDate p)) (value_ ropts)
+      _        -> \p -> maybe id (mixedAmountApplyValuation priceoracle styles (postingperiodend p) (_rsDay rspec) (postingDate p)) (value_ ropts)
+    costfn = case calc of
+      CalcGain -> id
+      _        -> journalToCost costop where costop = fromMaybe NoConversionOp $ conversionop_ ropts
 
     -- Find the end of the period containing this posting
     postingperiodend  = addDays (-1) . fromMaybe err . mPeriodEnd . postingDateOrDate2 (whichDate ropts)
@@ -623,7 +636,7 @@ mixedAmountApplyValuationAfterSumFromOptsWith ropts j priceoracle =
     gain mc spn = mixedAmountGainAtDate priceoracle styles mc (maybe err (addDays (-1)) $ spanEnd spn)
     costing = case fromMaybe NoConversionOp $ conversionop_ ropts of
         NoConversionOp -> id
-        ToCost         -> styleMixedAmount styles . mixedAmountCost
+        ToCost         -> styleAmounts styles . mixedAmountCost
     styles = journalCommodityStyles j
     err = error "mixedAmountApplyValuationAfterSumFromOptsWith: expected all spans to have an end date"
 
