@@ -9,7 +9,7 @@ The colour scheme may be somewhat hard-coded.
 
 -}
 
-{-# LANGUAGE CPP, LambdaCase #-}
+{-# LANGUAGE CPP, LambdaCase, PackageImports #-}
 
 module Hledger.Utils.IO (
 
@@ -80,6 +80,7 @@ module Hledger.Utils.IO (
   expandGlob,
   sortByModTime,
   readFileOrStdinPortably,
+  readFileStrictly,
   readFilePortably,
   readHandlePortably,
   -- hereFileRelative,
@@ -91,6 +92,7 @@ module Hledger.Utils.IO (
   )
 where
 
+import qualified Control.Exception as C (evaluate)
 import           Control.Monad (when, forM)
 import           Data.Colour.RGBSpace (RGB(RGB))
 import           Data.Colour.RGBSpace.HSL (lightness)
@@ -125,7 +127,7 @@ import           Text.Pretty.Simple
   defaultOutputOptionsDarkBg, defaultOutputOptionsNoColor, pShowOpt, pPrintOpt)
 
 import Hledger.Utils.Text (WideBuilder(WideBuilder))
-import System.FilePath.Glob (glob)
+import "Glob" System.FilePath.Glob (glob)
 import Data.Functor ((<&>))
 
 -- Pretty showing/printing with pretty-simple
@@ -461,6 +463,10 @@ sortByModTime fs = do
   ftimes <- forM fs $ \f -> do {t <- getModificationTime f; return (t,f)}
   return $ map snd $ reverse $ sort ftimes
 
+-- | Like readFilePortably, but read all of the file before proceeding.
+readFileStrictly :: FilePath -> IO T.Text
+readFileStrictly f = readFilePortably f >>= \t -> C.evaluate (T.length t) >> return t
+
 -- | Read text from a file,
 -- converting any \r\n line endings to \n,,
 -- using the system locale's text encoding,
